@@ -631,8 +631,9 @@ STDMETHODIMP CTextDocument::get_CodePage(long* pnCodePage) {
 STDMETHODIMP CTextDocument::get_EndPoint(ITextPoint** ppEndPoint) {
 	VERIFY_POINTER(ppEndPoint);
 	*ppEndPoint = new CTextPoint(m_pApp,
-		m_pDocument->GetController()->GetActiveView()->CreateEditPoint(CCharPos(-1, -1)));
+		m_pDocument->GetController()->GetActiveView()->CreateEditPoint());
 	(*ppEndPoint)->AddRef();
+	(*ppEndPoint)->MoveToEndOfDocument();
 	return S_OK;
 }
 
@@ -668,7 +669,7 @@ STDMETHODIMP CTextDocument::get_ReadOnly(VARIANT_BOOL* pbReadOnly) {
 STDMETHODIMP CTextDocument::get_StartPoint(ITextPoint** ppStartPoint) {
 	VERIFY_POINTER(ppStartPoint);
 	*ppStartPoint = new CTextPoint(m_pApp,
-		m_pDocument->GetController()->GetActiveView()->CreateEditPoint(CCharPos(0, 0)));
+		m_pDocument->GetController()->GetActiveView()->CreateEditPoint());
 	(*ppStartPoint)->AddRef();
 	return S_OK;
 }
@@ -894,7 +895,7 @@ STDMETHODIMP CTextProcessor::FindText(
 
 	if(bstrFindWhat == 0)
 		return E_INVALIDARG;
-	*pbFound = toVariantBoolean(m_pView->FindNext(bstrFindWhat, static_cast<EditViewFindFlag>(ftf)));
+	*pbFound = toVariantBoolean(m_pView->FindNext(bstrFindWhat, ftf));
 	return S_OK;
 }
 
@@ -905,10 +906,10 @@ STDMETHODIMP CTextProcessor::get_Application(IApplication** ppApplication) {
 	return S_OK;
 }
 
-///	@see	ITextProcessor::get_OverwriteMode
-STDMETHODIMP CTextProcessor::get_OverwriteMode(VARIANT_BOOL* pbOverwriteMode) {
-	VERIFY_POINTER(pbOverwriteMode);
-	*pbOverwriteMode = toVariantBoolean(m_pView->IsOverwriteMode());
+///	@see	ITextProcessor::get_OvertypeMode
+STDMETHODIMP CTextProcessor::get_OvertypeMode(VARIANT_BOOL* pbOvertypeMode) {
+	VERIFY_POINTER(pbOvertypeMode);
+	*pbOvertypeMode = toVariantBoolean(m_pView->IsOvertypeMode());
 	return S_OK;
 }
 
@@ -947,9 +948,9 @@ STDMETHODIMP CTextProcessor::Invoke(DISPID dispidMember,
 		case DISPID_TEXTPROCESSOR_APPLICATION:
 			pVarResult->vt = VT_DISPATCH;
 			return get_Application(reinterpret_cast<IApplication**>(&pVarResult->pdispVal));
-		case DISPID_TEXTPROCESSOR_OVERWRITEMODE:
+		case DISPID_TEXTPROCESSOR_OVERTYPEMODE:
 			pVarResult->vt = VT_BOOL;
-			return get_OverwriteMode(&pVarResult->boolVal);
+			return get_OvertypeMode(&pVarResult->boolVal);
 		case DISPID_TEXTPROCESSOR_SELECTION:
 			pVarResult->vt = VT_DISPATCH;
 			return get_Selection(reinterpret_cast<ITextSelection**>(&pVarResult->pdispVal));
@@ -959,10 +960,10 @@ STDMETHODIMP CTextProcessor::Invoke(DISPID dispidMember,
 		}
 	} else if(wFlags & DISPATCH_PROPERTYPUT)	// vb^
 		switch(dispidMember) {
-		case DISPID_TEXTPROCESSOR_OVERWRITEMODE:
+		case DISPID_TEXTPROCESSOR_OVERTYPEMODE:
 			VERIFY_ARGUMENTS_COUNT(1);
 			COERCE_ARGUMENT_AT(0, VT_BOOL);
-			return put_OverwriteMode(args[0].boolVal);
+			return put_OvertypeMode(args[0].boolVal);
 		case DISPID_TEXTPROCESSOR_TABWIDTH:
 			VERIFY_ARGUMENTS_COUNT(1);
 			COERCE_ARGUMENT_AT(0, VT_I2);
@@ -1017,9 +1018,9 @@ STDMETHODIMP CTextProcessor::Paste() {
 	return S_OK;
 }
 
-///	@see	ITextProcessor::put_OverwriteMode
-STDMETHODIMP CTextProcessor::put_OverwriteMode(VARIANT_BOOL bOverwriteMode) {
-	m_pView->SetOverwriteMode(toBoolean(bOverwriteMode));
+///	@see	ITextProcessor::put_OvertypeMode
+STDMETHODIMP CTextProcessor::put_OvertypeMode(VARIANT_BOOL bOvertypeMode) {
+	m_pView->SetOvertypeMode(toBoolean(bOvertypeMode));
 	return S_OK;
 }
 
@@ -1093,20 +1094,20 @@ STDMETHODIMP CTextSelection::get_ActiveEndGreater(VARIANT_BOOL* pbActiveEndGreat
 ///	@see	ITextSelection::get_ActivePoint
 STDMETHODIMP CTextSelection::get_ActivePoint(ITextPoint** ppActivePoint) {
 	VERIFY_POINTER(ppActivePoint);
-	CCharPos	posBegin, posEnd;
-	m_pView->GetSel(posBegin, posEnd);
-	*ppActivePoint = new CTextPoint(m_pApp, m_pView->CreateEditPoint(posEnd));
+	CCharPos	posActive = m_pView->GetActivePoint();
+	*ppActivePoint = new CTextPoint(m_pApp, m_pView->CreateEditPoint());
 	(*ppActivePoint)->AddRef();
+	(*ppActivePoint)->MoveTo(posActive.m_iLine, posActive.m_iChar);
 	return S_OK;
 }
 
 ///	@see	ITextSelection::get_AnchorPoint
 STDMETHODIMP CTextSelection::get_AnchorPoint(ITextPoint** ppAnchorPoint) {
 	VERIFY_POINTER(ppAnchorPoint);
-	CCharPos	posBegin, posEnd;
-	m_pView->GetSel(posBegin, posEnd);
-	*ppAnchorPoint = new CTextPoint(m_pApp, m_pView->CreateEditPoint(posBegin));
+	CCharPos	posAnchor = m_pView->GetAnchorPoint();
+	*ppAnchorPoint = new CTextPoint(m_pApp, m_pView->CreateEditPoint());
 	(*ppAnchorPoint)->AddRef();
+	(*ppAnchorPoint)->MoveTo(posAnchor.m_iLine, posAnchor.m_iChar);
 	return S_OK;
 }
 
@@ -1119,11 +1120,12 @@ STDMETHODIMP CTextSelection::get_Application(IApplication** ppApplication) {
 ///	@see	ITextSelection::get_BottomPoint
 STDMETHODIMP CTextSelection::get_BottomPoint(ITextPoint** ppBottomPoint) {
 	VERIFY_POINTER(ppBottomPoint);
-	CCharPos	posBegin, posEnd;
-	m_pView->GetSel(posBegin, posEnd);
-	*ppBottomPoint = new CTextPoint(
-		m_pApp, m_pView->CreateEditPoint((posBegin < posEnd) ? posEnd : posBegin));
-	(*ppBottomPoint)->AddRef(); 
+	CCharPos	posAnchor, posActive;
+	m_pView->GetSel(posAnchor, posActive);
+	posActive = max(posAnchor, posActive);
+	*ppBottomPoint = new CTextPoint(m_pApp, m_pView->CreateEditPoint());
+	(*ppBottomPoint)->AddRef();
+	(*ppBottomPoint)->MoveTo(posActive.m_iLine, posActive.m_iChar);
 	return S_OK;
 }
 
@@ -1151,11 +1153,12 @@ STDMETHODIMP CTextSelection::get_TextRanges(ITextRanges** ppTextRanges) {
 ///	@see	ITextSelection::get_TopPoint
 STDMETHODIMP CTextSelection::get_TopPoint(ITextPoint** ppTopPoint) {
 	VERIFY_POINTER(ppTopPoint);
-	CCharPos	posBegin, posEnd;
-	m_pView->GetSel(posBegin, posEnd);
-	*ppTopPoint = new CTextPoint(
-		m_pApp, m_pView->CreateEditPoint((posBegin < posEnd) ? posBegin : posEnd));
-	(*ppTopPoint)->AddRef(); 
+	CCharPos	posAnchor, posActive;
+	m_pView->GetSel(posAnchor, posActive);
+	posActive = min(posAnchor, posActive);
+	*ppTopPoint = new CTextPoint(m_pApp, m_pView->CreateEditPoint());
+	(*ppTopPoint)->AddRef();
+	(*ppTopPoint)->MoveTo(posActive.m_iLine, posActive.m_iChar);
 	return S_OK;
 }
 
@@ -1936,8 +1939,9 @@ STDMETHODIMP CTextPoint::WordPrev(long nWords /* = 1 */) {
 /////////////////////////////////////////////////////////////////////////////
 
 ///	RXgN^
-CTextRange::CTextRange(CAlphaApp* pApp, CEditRange* pRange) : m_pApp(pApp), m_pRange(pRange) {
-	assert(pApp != 0 && pRange != 0);
+CTextRange::CTextRange(CAlphaApp* pApp, CEditPoint* pPoint1, CEditPoint* pPoint2)
+		: m_pApp(pApp), m_pPoint1(pPoint1), m_pPoint2(pPoint2) {
+	assert(pApp != 0 && pPoint1 != 0 && pPoint2 != 0);
 }
 
 ///	@see	ITextRange::get_Application
@@ -1952,7 +1956,7 @@ STDMETHODIMP CTextRange::get_EndPoint(ITextPoint** ppEndPoint) {
 	VERIFY_POINTER(ppEndPoint);
 //	*ppEndPoint = new CTextPoint(m_pApp, m_pRange->GetEndPoint());
 //	(*ppEndPoint)->AddRef();
-	return S_OK;
+	return E_NOTIMPL;
 }
 
 ///	@see	ITextRange::get_StartPoint
@@ -1960,7 +1964,7 @@ STDMETHODIMP CTextRange::get_StartPoint(ITextPoint** ppStartPoint) {
 	VERIFY_POINTER(ppStartPoint);
 //	*ppStartPoint = new CTextPoint(m_pApp, m_pRange->GetStartPoint());
 //	(*ppStartPoint)->AddRef();
-	return S_OK;
+	return E_NOTIMPL;
 }
 
 ///	@see	ITextRange::Invoke
