@@ -4,7 +4,7 @@
 #include "StdAfx.h"
 #include "AlphaDoc.h"
 #include "AlphaView.h"
-#include "AlphaApplicationObject.h"
+#include "Ambient.h"
 using Alpha::CAlphaDoc;
 using Alpha::CAlphaView;
 
