@@ -4,6 +4,7 @@
 #include "StdAfx.h"
 #include "AutoCompleteWnd.h"
 #include "EditView.h"	// CEditView
+#include "EditPoint.h"
 
 using namespace Ascension;
 using namespace std;
@@ -24,16 +25,20 @@ void CAutoCompleteWnd::Complete() {
 	int	iSel = GetSelectionMark();
 
 	if(iSel != -1 && GetItemState(iSel, LVIS_SELECTED) == LVIS_SELECTED) {
-		wchar_t	wszItem[300];
-		wstring	strLeftWord = m_pView->GetPrecedingWord(250);
+		CEditDoc*		pDoc = m_pView->GetDocument();
+		wchar_t			wszItem[300];
+		const wstring	strLeftWord = m_pView->GetPrecedingWord(250);
 
 		GetItemText(iSel, 0, wszItem, 300);
-		if(m_pView->m_posAnchor < m_pView->m_posActive)
-			m_pView->m_posAnchor.m_iChar -= strLeftWord.length();
+		if(*m_pView->m_pAnchorPoint < *m_pView->m_pActivePoint)
+			m_pView->m_pAnchorPoint->CharPrev(strLeftWord.length());
 		else
-			m_pView->m_posActive.m_iChar -= strLeftWord.length();
-		m_pView->GetDocument()->DeleteText(m_pView, m_pView->m_posAnchor, m_pView->m_posActive, OCF_PREVENTCONCATFORWARD);
-		m_pView->GetDocument()->InsertText(m_pView, m_pView->m_posActive, wszItem, OCF_PREVENTCONCATBACKWARD);
+			m_pView->m_pActivePoint->CharPrev(strLeftWord.length());
+		pDoc->EndEditCollection();
+		pDoc->BeginEditCollection();
+		pDoc->DeleteText(m_pView, *m_pView->m_pAnchorPoint, *m_pView->m_pActivePoint);
+		pDoc->InsertText(m_pView, *m_pView->m_pActivePoint, wszItem);
+		pDoc->EndEditCollection();
 	}
 	ShowWindow(SW_HIDE);
 }
