@@ -1,17 +1,18 @@
 // EditDoc.cpp
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #include "StdAfx.h"
 #include "EditDoc.h"
 #include <MAPI.h>	// MAPISendMail
-#include "..\..\Manah\TextConverter.h"
+#include "..\..\Manah\Text.h"
 #include "..\..\Manah\WaitCursor.h"
-#include "..\..\Armaiti\ArmaitiImplementation.h"
+#include "..\..\Armaiti\ComGeneric.h"
 
 using namespace Ascension;
 using Ascension::CEditDoc::CUndoManager;
 using namespace Manah;
 using namespace Manah::Windows;
+using namespace Manah::Text;
 using namespace Armaiti;
 using namespace std;
 
@@ -25,8 +26,9 @@ CInsertOperation::CInsertOperation(const CCharPos& pos,
 }
 
 ///	폜͏ɌłȂ
-bool CInsertOperation::Concat(COperationUnit* pOperationUnit) {
-	return false;
+void CInsertOperation::Concat(COperationUnit* pOperationUnit) {
+	assert(pOperationUnit != 0);
+	pOperationUnit->Push(this);
 }
 
 ///	}̍Đ
@@ -38,7 +40,7 @@ void CInsertOperation::Execute(CEditDoc* pDoc) {
 		if(::GetFocus() == pView->m_hWnd)
 			break;
 	}
-	pDoc->InsertText(pView, m_pos, m_strText, OCF_VIRTUAL);
+	pDoc->InsertText(pView, m_pos, m_strText);
 }
 
 
@@ -50,11 +52,8 @@ CDeleteOperation::CDeleteOperation(const CCharPos& posBegin,
 		const CCharPos& posEnd) : m_bType(false), m_posBegin(posBegin), m_posEnd(posEnd) {
 }
 
-/**
- *	̑삪w肳ꂽ̌ɌB
- *	łȂ false Ԃ
- */
-bool CDeleteOperation::Concat(COperationUnit* pOperationUnit) {
+///	̑w肳ꂽPʂɌ
+void CDeleteOperation::Concat(COperationUnit* pOperationUnit) {
 	assert(pOperationUnit != 0);
 	IOperation*	pLastOpe = pOperationUnit->Top();
 
@@ -62,15 +61,11 @@ bool CDeleteOperation::Concat(COperationUnit* pOperationUnit) {
 		CDeleteOperation*	pPrevOperation = static_cast<CDeleteOperation*>(pLastOpe);
 		if(m_posBegin.m_iLine != pPrevOperation->m_posEnd.m_iLine
 				|| m_posBegin.m_iChar != pPrevOperation->m_posEnd.m_iChar)
-			return false;
-		pPrevOperation->m_posEnd = m_posEnd;	// Ō̑Pʂ̍Ō̑Ɍ (폜͈͂g)
-	} else {	// O͍폜
-		CInsertOperation*	pPrevOperation = static_cast<CInsertOperation*>(pLastOpe);
-		if(m_posBegin != pPrevOperation->m_pos)
-			return false;
+			pOperationUnit->Push(this);
+		else
+			pPrevOperation->m_posEnd = m_posEnd;	// Ō̑Pʂ̍Ō̑Ɍ (폜͈͂g)
+	} else	// O͍폜
 		pOperationUnit->Push(this);	// Ō̑Pʂɒǉ
-	}
-	return true;
 }
 
 ///	폜̍Đ
@@ -82,17 +77,13 @@ void CDeleteOperation::Execute(CEditDoc* pDoc) {
 		if(::GetFocus() == pView->m_hWnd)
 			break;
 	}
-	pDoc->DeleteText(pView, m_posBegin, m_posEnd, OCF_VIRTUAL);
+	pDoc->DeleteText(pView, m_posBegin, m_posEnd);
 }
 
 
 // COperationUnit class implementation
 /////////////////////////////////////////////////////////////////////////////
 
-/// RXgN^
-COperationUnit::COperationUnit(bool bPreventConcat) : m_bPreventConcat(bPreventConcat) {
-}
-
 /// fXgN^
 COperationUnit::~COperationUnit() {
 	while(!m_stkOpes.empty()) {
@@ -101,11 +92,6 @@ COperationUnit::~COperationUnit() {
 	}
 }
 
-///	̑Pʂ㑱̑Ƃ̌F߂邩ǂԂ
-bool COperationUnit::DoesPermitConcat() const {
-	return !m_bPreventConcat;
-}
-
 /**
  *	Pʂ̎sB\bhďoケ̃IuWFNg͖ɂȂ
  *	@param pDoc	ΏۃhLg
@@ -185,19 +171,11 @@ size_t CUndoManager::GetUndoBufferLength() const {
 }
 
 /**
- *	1̑AhDX^bNɒǉB
- *	Oǉꂽ1܂Ƃ߂ɂłꍇ1̑PʂƂČB
- *	<var>ocf</var>  <code>OCF_TYPING</code> ɂƂ͂̃\bh邩ǂ߂B
- *	ł͈̂ȉ̏ꍇ̂
- *	<ul>
- *		<li>O񂪍폜ō񂪑}AJnʒu (u)</li>
- *		<li>O}ŁAȌIʒůJnʒuƓ</li>
- *	</ul>
- *	@param pOperation		ǉ鑀
- *	@param ocf				ȎƂ̌邩ǂ̃tO
- *	@see					OperationConcatenationFlag, IOperation::CanFollow
+ *	1̑AhDX^bNɒǉ
+ *	@param pOperation	ǉ鑀
+ *	@param bConcatPrev	Ȏƌ邩ǂ
  */
-void CUndoManager::PushUndoBuffer(IOperation* pOperation, OperationConcatenationFlag ocf) {
+void CUndoManager::PushUndoBuffer(IOperation* pOperation, bool bConcatPrev) {
 	// hDX^bNɂ
 	if(!m_bVirtual) {
 		while(!m_stkRedo.empty()) {
@@ -208,25 +186,16 @@ void CUndoManager::PushUndoBuffer(IOperation* pOperation, OperationConcatenation
 
 	if(m_bVirtual) {	// z쎞̓X^bNւ̒ǉx
 		if(m_pVirtualUnit == 0)	// 
-			m_pVirtualUnit = new COperationUnit(false);
+			m_pVirtualUnit = new COperationUnit();
 		m_pVirtualUnit->Push(pOperation);
+	} else if(bConcatPrev && m_pLastUnit) {	// Ō̑PʂɌ
+		assert(m_pLastUnit != 0);
+		m_pLastUnit->Push(pOperation);
 	} else {
-		// \ׂ
-		if(m_pLastUnit != 0
-				&& !(ocf & OCF_TYPING)
-				&& !(ocf & OCF_PREVENTCONCATFORWARD)
-				&& m_pLastUnit->DoesPermitConcat())	// 
-			m_pLastUnit->Push(pOperation);
-		else if(ocf & OCF_PREVENTCONCATFORWARD
-				|| m_pLastUnit == 0
-				|| !m_pLastUnit->DoesPermitConcat()
-				|| ocf != OCF_TYPING
-				|| !pOperation->Concat(m_pLastUnit)) {
-			COperationUnit*	pUnit = new COperationUnit(toBoolean(ocf & OCF_PREVENTCONCATBACKWARD));
-			pUnit->Push(pOperation);
-			m_stkUndo.push(pUnit);
-			m_pLastUnit = pUnit;
-		}
+		COperationUnit*	pUnit = new COperationUnit();
+		pUnit->Push(pOperation);
+		m_stkUndo.push(pUnit);
+		m_pLastUnit = pUnit;
 	}
 }
 
@@ -273,7 +242,7 @@ map<UINT, CEditDoc*> CEditDoc::m_mapDocuments;
 ///	RXgN^
 CEditDoc::CEditDoc(Ascension::CEditController* pController /* = 0 */) :
 		Manah::Windows::CDocument((CController*)pController), m_bIgnoreViews(false), m_bReadOnly(false),
-		m_fomMode(FOM_DENYNONE), m_breakType(BT_CRLF),
+		m_fomMode(FOM_DENYNONE), m_breakType(BT_CRLF), m_ugsGroupingState(UGS_NONE),
 		m_bOnceUndoBufferCleared(false), m_bVirtualOperating(false), m_pEventListener(0),
 		m_itCache_(m_listLines.end()), m_itCache(m_listLines.end()) {
 	CEditDocLine	oLine;
@@ -294,6 +263,16 @@ CEditDoc::~CEditDoc() {
 		CEditDoc::m_mapDocuments.erase(it);
 }
 
+///	AhDO[v̎WJnB
+///	ݎWł΁AxIAVWJn
+void CEditDoc::BeginEditCollection() {
+	AssertValid();
+
+	if(m_ugsGroupingState != UGS_NONE)
+		EndEditCollection();
+	m_ugsGroupingState = UGS_WAITFORFIRSTEDIT;
+}
+
 ///	t@C̍ŏIXV`FbN
 void CEditDoc::CheckFileLastWriteTime() {
 	AssertValid();
@@ -440,13 +419,11 @@ OperationStatus CEditDoc::DeleteCurrentFile() {
  *	@param pSender	hLgɏvr[
  *	@param posBegin	폜Jn_
  *	@param posEnd	폜I_
- *	@param ocf		쌋tO
  *	@return			IɃLbguʒu
  *	@throw EDocumentIsReadOnly	ǂݎp̂ƂX[
  */
-CCharPos CEditDoc::DeleteText(CView* pSender,
-		const CCharPos& posBegin, const CCharPos& posEnd,
-		OperationConcatenationFlag ocf) throw(EDocumentIsReadOnly) {
+CCharPos CEditDoc::DeleteText(CObject* pSender,
+		const CCharPos& posBegin, const CCharPos& posEnd) throw(EDocumentIsReadOnly) {
 	AssertValid();
 
 	if(m_bReadOnly)
@@ -501,13 +478,16 @@ CCharPos CEditDoc::DeleteText(CView* pSender,
 	m_itCache = m_listLines.end();
 
 	m_bModified = true;
-	m_pUndoManager->PushUndoBuffer(new CInsertOperation(posABegin, strDeleted), ocf);
+	m_pUndoManager->PushUndoBuffer(new CInsertOperation(
+		posABegin, strDeleted), m_ugsGroupingState == UGS_WAITFORCONTINUEEDIT);
+	if(m_ugsGroupingState == UGS_WAITFORFIRSTEDIT)
+		m_ugsGroupingState = UGS_WAITFORCONTINUEEDIT;
 
 	if(!m_bIgnoreViews) {
 		// Ō̍XVXV
-		m_oUpdateInfo.bIsInsert = false;
-		m_oUpdateInfo.posBegin = min(posBegin, posEnd);
-		m_oUpdateInfo.posEnd = max(posBegin, posEnd);
+		m_oUpdateInfo.usSummary = US_OPERATION_DELETE;
+		m_oUpdateInfo.posBegin = posABegin;
+		m_oUpdateInfo.posEnd = posAEnd;
 		m_oUpdateInfo.posResult = posABegin;
 		UpdateAllViews(pSender, 0, this);
 	}
@@ -517,6 +497,12 @@ CCharPos CEditDoc::DeleteText(CView* pSender,
 	return posABegin;
 }
 
+///	AhDO[v̎WI
+void CEditDoc::EndEditCollection() {
+	AssertValid();
+	m_ugsGroupingState = UGS_NONE;
+}
+
 /**
  *	Ss (hLgS) 擾
  *	@param strText	i[
@@ -681,6 +667,7 @@ void CEditDoc::InitiateDocument() {
 	m_itCache = m_listLines.end();
 	m_bModified = false;
 	ClearUndoBuffer();
+	m_ugsGroupingState = UGS_NONE;
 	m_bOnceUndoBufferCleared = false;
 
 	if(m_pEventListener != 0)
@@ -697,13 +684,11 @@ void CEditDoc::InitiateDocument() {
  *	@param pSender	̏vr[
  *	@param pos		}ʒu
  *	@param strText	}镶
- *	@param ocf		쌋tO
  *	@return			ɃLbguʒu
  *	@throw EDocumentIsReadOnly	ǂݎp̂ƂX[
  */
-CCharPos CEditDoc::InsertText(CView* pSender,
-		const CCharPos& pos, const wstring& strText,
-		OperationConcatenationFlag ocf) throw(EDocumentIsReadOnly) {
+CCharPos CEditDoc::InsertText(CObject* pSender,
+		const CCharPos& pos, const wstring& strText) throw(EDocumentIsReadOnly) {
 	AssertValid();
 
 	if(m_bReadOnly)
@@ -785,13 +770,16 @@ CCharPos CEditDoc::InsertText(CView* pSender,
 	}
 
 	m_bModified = true;
-	m_pUndoManager->PushUndoBuffer(new CDeleteOperation(pos, posResult), ocf);
+	m_pUndoManager->PushUndoBuffer(new CDeleteOperation(
+		pos, posResult), m_ugsGroupingState == UGS_WAITFORCONTINUEEDIT);
+	if(m_ugsGroupingState == UGS_WAITFORFIRSTEDIT)
+		m_ugsGroupingState = UGS_WAITFORCONTINUEEDIT;
 
 	if(m_pEventListener != 0)
 		m_pEventListener->OnDocumentModified();
 	if(!m_bIgnoreViews) {
 		// Ō̍XVXV
-		m_oUpdateInfo.bIsInsert = true;
+		m_oUpdateInfo.usSummary = US_OPERATION_INSERT;
 		m_oUpdateInfo.posBegin = pos;
 		m_oUpdateInfo.posEnd = pos;
 		m_oUpdateInfo.posResult = posResult;
@@ -852,11 +840,13 @@ StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
 		m_oFile.Close();
 		m_bReadOnly = false;
 		m_fomMode = fom;
-		if(fom == FOM_ASREADONLY)
+		if(fom == FOM_ASREADONLY) {
 			m_oFile.Open(wszPathName, CFile::modeRead | CFile::shareDenyNone);
-		else if(!m_oFile.Open(wszPathName, CFile::modeReadWrite | CFile::shareDenyNone, false)) {
+			m_bReadOnly = true;
+		} else if(!m_oFile.Open(wszPathName, CFile::modeReadWrite | CFile::shareDenyNone, false)) {
 			ss = SS_READ_READONLY;
 			m_oFile.Open(wszPathName, CFile::modeRead | CFile::shareDenyNone);
+			m_bReadOnly = true;
 		}
 		m_oFile.GetFileTime(0, 0, &m_lastWriteTime);
 		dwFileSize = m_oFile.GetFileSize(0);	// 4GB ̃XNvgȂ񂩖낤
@@ -876,61 +866,61 @@ StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
 	m_oFile.Close();
 
 	// R[hϊ
-	CTextConverter::CharCode	cc;
+	CharCode	cc;
 	if(dwFileSize != 0) {
 		hGlobal2 = ::GlobalAlloc(GMEM_MOVEABLE, sizeof(wchar_t) * dwRead * 3);	// ȃTCY...
 		pwszBuffer = static_cast<wchar_t*>(::GlobalLock(hGlobal2));
 		switch(nCodePage) {
 		case EXTCP_AUTO:
-			cc = CTextConverter::CC_AUTO;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_AUTO;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			switch(cc) {
-			case CTextConverter::CC_UTF16LE:	nCodePage = EXTCP_UTF16LE;	break;
-			case CTextConverter::CC_UTF16BE:	nCodePage = EXTCP_UTF16BE;	break;
-			case CTextConverter::CC_UTF32LE:	nCodePage = EXTCP_UTF32LE;	break;
-			case CTextConverter::CC_UTF32BE:	nCodePage = EXTCP_UTF32BE;	break;
-			case CTextConverter::CC_UTF8:		nCodePage = CP_UTF8;		break;
-			case CTextConverter::CC_UTF7:		nCodePage = EXTCP_UTF5;		break;
-			case CTextConverter::CC_SHIFTJIS:	nCodePage = 932;			break;
-			case CTextConverter::CC_JIS:		nCodePage = 50221;			break;
-			case CTextConverter::CC_EUCJP:		nCodePage = EXTCP_EUCJP;	break;
+			case CC_UTF16LE:	nCodePage = EXTCP_UTF16LE;	break;
+			case CC_UTF16BE:	nCodePage = EXTCP_UTF16BE;	break;
+			case CC_UTF32LE:	nCodePage = EXTCP_UTF32LE;	break;
+			case CC_UTF32BE:	nCodePage = EXTCP_UTF32BE;	break;
+			case CC_UTF8:		nCodePage = CP_UTF8;		break;
+			case CC_UTF7:		nCodePage = EXTCP_UTF5;		break;
+			case CC_SHIFTJIS:	nCodePage = 932;			break;
+			case CC_JIS:		nCodePage = 50221;			break;
+			case CC_EUCJP:		nCodePage = EXTCP_EUCJP;	break;
 			}
 			break;
 		case EXTCP_UTF16LE:
-			cc = CTextConverter::CC_UTF16LE;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_UTF16LE;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			break;
 		case EXTCP_UTF16BE:
-			cc = CTextConverter::CC_UTF16BE;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_UTF16BE;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			break;
 		case EXTCP_UTF32LE:
-			cc = CTextConverter::CC_UTF32LE;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_UTF32LE;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			break;
 		case EXTCP_UTF32BE:
-			cc = CTextConverter::CC_UTF32BE;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_UTF32BE;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			break;
 		case CP_UTF8:
-			cc = CTextConverter::CC_UTF8;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_UTF8;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			break;
 		case EXTCP_UTF5:
-			cc = CTextConverter::CC_UTF5;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_UTF5;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			break;
 		case 932:	// Shift-JIS
-			cc = CTextConverter::CC_SHIFTJIS;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_SHIFTJIS;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			break;
 		case 50221:	// JIS
-			cc = CTextConverter::CC_JIS;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_JIS;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			break;
 		case EXTCP_EUCJP:
-			cc = CTextConverter::CC_EUCJP;
-			dwRead = CTextConverter::ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
+			cc = CC_EUCJP;
+			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
 			break;
 		default:
 			dwRead = ::MultiByteToWideChar(nCodePage, 0,
@@ -969,7 +959,7 @@ StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
 		nCodePage = ::GetACP();
 		m_listLines.clear();
 		m_listLines.push_back(CEditDocLine());
-		InsertText(0, CCharPos(0, 0), L"", OCF_CONCATALL);
+		InsertText(0, CCharPos(0, 0), L"");
 		m_listLines.begin()->m_cOperationHistory = 0;
 	}
 
@@ -1015,6 +1005,7 @@ StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
 	m_nCodePage = nCodePage;
 	ClearUndoBuffer();
 	m_bModified = false;
+	m_ugsGroupingState = UGS_NONE;
 	m_bOnceUndoBufferCleared = false;
 	m_strPathName = wszPathName;
 	m_strTitle = ::PathFindFileNameW(wszPathName);
@@ -1104,7 +1095,16 @@ void CEditDoc::Redo() throw(EDocumentIsReadOnly) {
 	AssertValid();
 	if(m_bReadOnly)
 		throw EDocumentIsReadOnly();
+	if(m_pUndoManager->GetRedoBufferLength() == 0)
+		return;
+
+	BeginEditCollection();
+	m_oUpdateInfo.usSummary = US_BEGIN_UNDOGROUP;
+	UpdateAllViews(this, 0, this);
 	m_pUndoManager->Redo();
+	EndEditCollection();
+	m_oUpdateInfo.usSummary = US_END_UNDOGROUP;
+	UpdateAllViews(this, 0, this);
 }
 
 /**
@@ -1208,7 +1208,7 @@ StreamStatus CEditDoc::SaveDocument(const wstring& strPathName,
 	unsigned long				cchLine;				// R[hϊO̒
 	unsigned long				len;					// R[hϊ̒
 	wstring						strLine;				// 1s
-	CTextConverter::CharCode	cc;
+	CharCode					cc;
 
 	for(unsigned long iLine = 0; iLine < cLine; ++iLine, ++it) {
 		it->m_cOperationHistory = 0;	// 엚
@@ -1227,67 +1227,58 @@ StreamStatus CEditDoc::SaveDocument(const wstring& strPathName,
 			cchLine += (((bt == BT_AUTO) ? it->m_breakType : m_breakType) == BT_CRLF) ? 2 : 1;
 		switch(nCodePage) {
 		case EXTCP_UTF16LE:
-			cc = CTextConverter::CC_UTF16LE;
+			cc = CC_UTF16LE;
 			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
 			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = CTextConverter::ConvertUnicodeTo(cc, pszBuffer,
-					strLine.c_str(), 0, cchLine);
+			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
 			break;
 		case EXTCP_UTF16BE:
-			cc = CTextConverter::CC_UTF16BE;
+			cc = CC_UTF16BE;
 			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
 			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = CTextConverter::ConvertUnicodeTo(cc, pszBuffer,
-					strLine.c_str(), 0, cchLine);
+			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
 			break;
 		case CP_UTF8:
-			cc = CTextConverter::CC_UTF8;
+			cc = CC_UTF8;
 			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 3);
 			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = CTextConverter::ConvertUnicodeTo(cc, pszBuffer,
-					strLine.c_str(), 0, cchLine);
+			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
 			break;
 		case EXTCP_UTF32LE:
-			cc = CTextConverter::CC_UTF32LE;
+			cc = CC_UTF32LE;
 			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
 			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = CTextConverter::ConvertUnicodeTo(cc, pszBuffer,
-					strLine.c_str(), 0, cchLine);
+			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
 			break;
 		case EXTCP_UTF32BE:
-			cc = CTextConverter::CC_UTF32BE;
+			cc = CC_UTF32BE;
 			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
 			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = CTextConverter::ConvertUnicodeTo(cc, pszBuffer,
-					strLine.c_str(), 0, cchLine);
+			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
 			break;
 		case EXTCP_UTF5:
-			cc = CTextConverter::CC_UTF5;
+			cc = CC_UTF5;
 			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 4);
 			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = CTextConverter::ConvertUnicodeTo(cc, pszBuffer,
-					strLine.c_str(), 0, cchLine);
+			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
 			break;
 		case 932:	// Shift-JIS
-			cc = CTextConverter::CC_SHIFTJIS;
+			cc = CC_SHIFTJIS;
 			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 3);
 			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = CTextConverter::ConvertUnicodeTo(cc, pszBuffer,
-					strLine.c_str(), 0, cchLine);
+			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
 			break;
 		case EXTCP_EUCJP:
-			cc = CTextConverter::CC_EUCJP;
+			cc = CC_EUCJP;
 			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
 			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = CTextConverter::ConvertUnicodeTo(cc, pszBuffer,
-					strLine.c_str(), 0, cchLine);
+			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
 			break;
 		case 50221:	// JIS
-			cc = CTextConverter::CC_JIS;
+			cc = CC_JIS;
 			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 3);
 			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = CTextConverter::ConvertUnicodeTo(cc, pszBuffer,
-					strLine.c_str(), 0, cchLine);
+			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
 			break;
 		default:
 			len = ::WideCharToMultiByte(nCodePage, 0,
@@ -1330,7 +1321,9 @@ StreamStatus CEditDoc::SaveDocument(const wstring& strPathName,
 		::FindClose(hFind);
 	}
 
-	UpdateAllViews(0);
+	// r[ɒʒm
+	m_oUpdateInfo.usSummary = US_SAVEDOCUMENT;
+	UpdateAllViews(this, 0, this);
 
 	if(m_pEventListener != 0)
 		m_pEventListener->OnDocumentModified();
@@ -1495,9 +1488,20 @@ void CALLBACK CEditDoc::TimerProc(HWND hWnd, UINT nMsg, UINT_PTR idEvent, DWORD
  *	@throw EDocumentIsReadOnly	ǂݎp̂ƂX[
  */
 void CEditDoc::Undo() {
+	AssertValid();
 	if(m_bReadOnly)
 		throw EDocumentIsReadOnly();
+	if(m_pUndoManager->GetUndoBufferLength() == 0)
+		return;
+
+	BeginEditCollection();
+	m_oUpdateInfo.usSummary = US_BEGIN_UNDOGROUP;
+	UpdateAllViews(this, 0, this);
 	m_pUndoManager->Undo();
+	EndEditCollection();
+	m_oUpdateInfo.usSummary = US_END_UNDOGROUP;
+	UpdateAllViews(this, 0, this);
+
 	if(!m_bOnceUndoBufferCleared && m_pUndoManager->GetUndoBufferLength() == 0) {
 		m_bModified = false;
 	if(m_pEventListener != 0)
