@@ -1,12 +1,12 @@
 // EditDoc.h
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #ifndef _EDIT_DOC_H_
 #define _EDIT_DOC_H_
 
-#include <string>
 #include <stack>
 #include <map>
+#include "AscensionCommon.h"
 #include "..\..\Manah\DocumentView.h"
 #include "..\..\Manah\File.h"
 
@@ -16,60 +16,11 @@ namespace Ascension {
 class CEditDoc;
 class CEditController;
 
-// CCharPos class definition and implementation
-/////////////////////////////////////////////////////////////////////////////
-
-///	sƕʒui[
-class CCharPos {
-	// f[^o
-public:
-	unsigned long	m_iLine;	/// s
-	unsigned long	m_iChar;	/// ʒu
-
-	// RXgN^
-public:
-	///	RXgN^
-	explicit CCharPos(unsigned long iLine = 0,
-			unsigned long iChar = 0) : m_iLine(iLine), m_iChar(iChar) {
-	}
-
-	// Zq
-public:
-	bool operator ==(const CCharPos& pos) const {
-		return (m_iLine == pos.m_iLine && m_iChar == pos.m_iChar);
-	}
-	bool operator !=(const CCharPos& pos) const {
-		return (m_iLine != pos.m_iLine || m_iChar != pos.m_iChar);
-	}
-	bool operator <(const CCharPos& pos) const {
-		return (m_iLine < pos.m_iLine
-				|| (m_iLine == pos.m_iLine && m_iChar < pos.m_iChar));
-	}
-	bool operator <=(const CCharPos& pos) const {
-		return (*this < pos || *this == pos);
-	}
-	bool operator >(const CCharPos& pos) const {
-		return (m_iLine > pos.m_iLine
-				|| (m_iLine == pos.m_iLine && m_iChar > pos.m_iChar));
-	}
-	bool operator >=(const CCharPos& pos) const {
-		return (*this > pos || *this == pos);
-	}
-};
-
 
 // COperation interface definition
 /////////////////////////////////////////////////////////////////////////////
 
 class COperationUnit;
-// ҏW̎
-enum EditOperationType {
-	EOT_NONE,		// 
-	EOT_TYPING,		// ^CsO
-	EOT_DELETE,		// 폜
-	EOT_REPLACE,	// ㏑
-	EOT_PASTE,		// \t
-};
 
 /**
  *	炩̕ҏW
@@ -79,7 +30,7 @@ interface IOperation {
 	///	fXgN^
 	virtual ~IOperation() {}
 	///  <var>pOperationUnit</var> Ɍ
-	virtual bool	Concat(COperationUnit* pOperationUnit) = 0;
+	virtual void	Concat(COperationUnit* pOperationUnit) = 0;
 	/// s
 	virtual void	Execute(CEditDoc* pDoc) = 0;
 };
@@ -108,7 +59,7 @@ public:
 
 	// \bh
 public:
-	virtual bool	Concat(COperationUnit* pOperationUnit);
+	virtual void	Concat(COperationUnit* pOperationUnit);
 	virtual void	Execute(CEditDoc* pDoc);
 };
 
@@ -134,7 +85,7 @@ public:
 
 	// \bh
 public:
-	virtual bool	Concat(COperationUnit* pOperationUnit);
+	virtual void	Concat(COperationUnit* pOperationUnit);
 	virtual void	Execute(CEditDoc* pDoc);
 };
 
@@ -144,23 +95,20 @@ public:
 
 ///	܂Ƃ߂ăAhD/hD\ȈȂ
 class COperationUnit {
-	// f[^o
-private:
-	bool					m_bPreventConcat;	// 㑱̑̌ȂƂ
-	std::stack<IOperation*>	m_stkOpes;			// ̒PʂɊ܂܂Ȃ
-
 	// RXgN^
 public:
-	COperationUnit(bool bPreventConcat);
 	virtual ~COperationUnit();
 
 	// \bh
 public:
-	bool		DoesPermitConcat() const;
 	void		Execute(CEditDoc* pDoc);
 	IOperation*	Pop();
 	void		Push(IOperation* pOperation);
 	IOperation*	Top() const;
+
+	// f[^o
+private:
+	std::stack<IOperation*>	m_stkOpes;	// ̒PʂɊ܂܂Ȃ
 };
 
 
@@ -267,36 +215,30 @@ enum OperationStatus {
 	OPS_UNKNOWNERROR,
 };
 
-/// AhDobt@ɒǉƂ̃tO
-enum OperationConcatenationFlag {
-	/// ̑삪Ȏƌ̂
-	OCF_CONCATFORWARD			= 0x00,
-	/// ̑삪Ȏƌ̂֎~
-	OCF_PREVENTCONCATFORWARD	= 0x01,
-	/// ̑삪̑ƌ̂
-	OCF_CONCATBACKWARD			= 0x00,
-	/// ̑삪̑ƌ̂֎~
-	OCF_PREVENTCONCATBACKWARD	= 0x10,
-	/// ^CsO (CEditDoc::PushUndoBuffer ɂ)
-	OCF_TYPING					= 0x20,
-	/// z (p)
-	OCF_VIRTUAL					= 0x80,
-	///	Sċ (= OCF_CONCATFORWARD | OCF_CONCATBACKWARD)
-	OCF_CONCATALL				= 0x00,
-	///	Sċ֎~ (= OCF_PREVENTCONCATFORWARD | OCF_PREVENTCONCATBACKWARD)
-	OCF_PREVENTALL				= 0x11
+///	CEditView::OnUpdate ̍XV̎
+enum UpdateSummary {
+	///	}
+	US_OPERATION_INSERT,
+	///	폜
+	US_OPERATION_DELETE,
+	///	AhD/hD̊Jn
+	US_BEGIN_UNDOGROUP,
+	///	AhD/hD̏I
+	US_END_UNDOGROUP,
+	///	hLgۑ
+	US_SAVEDOCUMENT,
 };
 
 ///	CEditDoc::GetModification œXV
 struct TUpdateInfo {
-	///	}ł trueB폜ł false
-	bool		bIsInsert;
+	///	UpdateSummary Q
+	UpdateSummary	usSummary;
 	///	ΏۊJnʒu
-	CCharPos	posBegin;
+	CCharPos		posBegin;
 	///	ΏۏIʒu (posBegin &lt;= posEnd)
-	CCharPos	posEnd;
+	CCharPos		posEnd;
 	///	I̐Lbgʒu
-	CCharPos	posResult;
+	CCharPos		posResult;
 };
 
 ///	hLgǂݎp̂ƂҏW悤ƂƃX[
@@ -352,7 +294,6 @@ interface IEditDocEventListener {
 
 typedef std::list<CEditDocLine>				CEditDocLineList;
 typedef CEditDocLineList::const_iterator	LineIterator;
-typedef std::basic_string<unsigned long>	xstring;	// UTF-32 
 
 typedef unsigned short	SaveDocumentOption;
 const SaveDocumentOption	SDO_WRITE_BOM			= 0x0001;	///	BOM 
@@ -364,6 +305,18 @@ const SaveDocumentOption	SDO_IGNORE_NOFITCHARS	= 0x0002;	///	ΉȂɑ΂Ċ̕ւ
  *	SĂ̍s̓eA엚AR[hy[WAsR[hȂǂB
  *	t@Cs
  *
+ *	<h3>AhDO[v̊TO</h3>
+ *
+ *	Ascension GfB^ɂ́Ȃ܂Ƃ߂ăAhD/hDł悤
+ *	AhDO[vƂTOB^CsOɂA͂̈ꊇ
+ *	쑤̎dłBCEditView ACEditPoint ͂Ƃɂ̑Ă邽߁A
+ *	̃NXgăhLgҏWꍇA[U̓^CsOɂꊇ
+ *	̂߂̃R[hKv͖
+ *
+ *	̑AhDO[vƂɂ͈Ȃ̑O
+ *	CEditDoc::BeginEditCollection 1xĂяoBđ삪I
+ *	CEditDoc::EndEditCollection Ăяo
+ *
  *	@see	CEditView, CEditController, CEditPoint
  */
 class CEditDoc : public Manah::Windows::CDocument {
@@ -372,7 +325,7 @@ class CEditDoc : public Manah::Windows::CDocument {
 	/////////////////////////////////////////////////////////////////////////
 
 	///	AhDAhDǗ
-	class CUndoManager /*: public Manah::CManahObject*/ {
+	class CUndoManager /*: public Manah::CObject*/ {
 		// f[^o
 	private:
 		CEditDoc*					m_pDoc;			// ΏۃhLg
@@ -389,13 +342,20 @@ class CEditDoc : public Manah::Windows::CDocument {
 
 		// \bh
 	public:
-		void	Clear();								// obt@ɂ
-		size_t	GetRedoBufferLength() const;			// hD\ȉ񐔂Ԃ
-		size_t	GetUndoBufferLength() const;			// AhD\ȉ񐔂Ԃ
-		void	PushUndoBuffer(IOperation* pOperation,	// AhDobt@ɑǉ
-					OperationConcatenationFlag ocf);
-		void	Redo();									// hLgɃhDs
-		void	Undo();									// hLgɃAhDs
+		void	Clear();						// obt@ɂ
+		size_t	GetRedoBufferLength() const;	// hD\ȉ񐔂Ԃ
+		size_t	GetUndoBufferLength() const;	// AhD\ȉ񐔂Ԃ
+		void	PushUndoBuffer(					// AhDobt@ɑǉ
+					IOperation* pOperation, bool bConcatPrev);
+		void	Redo();							// hLgɃhDs
+		void	Undo();							// hLgɃAhDs
+	};
+
+	///	AhDO[v̏
+	enum UndoGroupingState {
+		UGS_NONE,				///	O[sO͍sĂȂ
+		UGS_WAITFORFIRSTEDIT,	///	O[sOJnҏW1xsĂȂ
+		UGS_WAITFORCONTINUEEDIT	///	O[sOJn1xȏҏWsꂽ
 	};
 
 	// f[^o
@@ -411,6 +371,7 @@ private:
 	Manah::Windows::IO::CFile	m_oFile;		// ݊JĂt@C
 	CUndoManager*				m_pUndoManager;	// AhD/hD̊Ǘ
 	TUpdateInfo					m_oUpdateInfo;	// Ō̍XV
+	UndoGroupingState			m_ugsGroupingState;			// AhDO[v̎W
 	bool						m_bOnceUndoBufferCleared;	// 1xȏAhDobt@NA
 
 	bool						m_bVirtualOperating;	// AhD/hDɂ DeleteText AInsertText
@@ -454,14 +415,14 @@ public:
 	void				SetReadOnly(bool bReadOnly = true);								// ǂݎp[h̐ݒ
 
 	// ҏW
-	CCharPos	DeleteText(Manah::Windows::CView* pSender,					// w͈͂폜
+	void		BeginEditCollection();
+	CCharPos	DeleteText(Manah::CObject* pSender,							// w͈͂폜
 					const CCharPos& posBegin,
-					const CCharPos& posEnd,
-					OperationConcatenationFlag ocf) throw(EDocumentIsReadOnly);
-	CCharPos	InsertText(Manah::Windows::CView* pSender,					// wʒuɕ}
+					const CCharPos& posEnd) throw(EDocumentIsReadOnly);
+	void		EndEditCollection();
+	CCharPos	InsertText(Manah::CObject* pSender,							// wʒuɕ}
 					const CCharPos& pos,
-					const std::wstring& strText,
-					OperationConcatenationFlag ocf) throw(EDocumentIsReadOnly);
+					const std::wstring& strText) throw(EDocumentIsReadOnly);
 	void		Redo() throw(EDocumentIsReadOnly);							// hD̎s
 	void		SetEventListener(IEditDocEventListener* pEventListener);	// CxgXi̐ݒ
 	void		Undo() throw(EDocumentIsReadOnly);							// AhD̎s
