@@ -1,5 +1,5 @@
-// (c) 2003 exeal
 // EditPoint.cpp
+// (c) 2003-2004 exeal
 
 #include "StdAfx.h"
 #include "EditPoint.h"
@@ -19,12 +19,15 @@ using namespace std;
  *	RXgN^
  *	@param pView	Ώۂ̃r[
  */
-CEditPoint::CEditPoint(CEditView* pView) : m_pView(pView) {
+CEditPoint::CEditPoint(CEditView* pView)
+		: m_iLine(0), m_iChar(0), m_pView(pView),
+		m_bSyncWithOtherEdit(false), m_bOwnedByView(false), m_cxLast(0) {
+	assert(m_pView != 0);
 }
 
 ///	fXgN^
 CEditPoint::~CEditPoint() {
-	if(m_pView != 0)
+	if(m_pView != 0 && m_bOwnedByView)
 		m_pView->ReleaseEditPoint(*this);
 }
 
@@ -52,9 +55,7 @@ void CEditPoint::CharNext(unsigned long nOffset /* = 1 */) throw(EViewIsAlreadyI
 			// Lbg󂯎Ȃ΂
 			m_iChar += CEditView::MakeCaretPosValid(pwszLine, strLine.length(), ++m_iChar, false);
 	}
-	m_cxLast = m_pView->PosFromChar(*this).x
-				+ m_pView->m_ptScroll.x * m_pView->m_layoutInfo.nHScrollRatio * m_pView->GetAvgCharWidth()
-				- m_pView->m_layoutInfo.nLeftTabWidth + 1;
+	UpdateLastCx();
 }
 
 /**
@@ -80,9 +81,7 @@ void CEditPoint::CharPrev(unsigned long nOffset /* = 1 */) throw(EViewIsAlreadyI
 			// Lbg󂯎Ȃ΂
 			m_iChar += CEditView::MakeCaretPosValid(pwszLine, strLine.length(), --m_iChar, true);
 	}
-	m_cxLast = m_pView->PosFromChar(*this).x
-				+ m_pView->m_ptScroll.x * m_pView->m_layoutInfo.nHScrollRatio * m_pView->GetAvgCharWidth()
-				- m_pView->m_layoutInfo.nLeftTabWidth + 1;
+	UpdateLastCx();
 }
 
 ///	ʒu\IuWFNgԂB߂l͌Ăяoō폜
@@ -93,7 +92,8 @@ CEditPoint* CEditPoint::Clone() const throw(EViewIsAlreadyInavailable) {
 	CEditPoint*	pClone = new CEditPoint(m_pView);
 	pClone->m_iLine = m_iLine;
 	pClone->m_iChar = m_iChar;
-	m_pView->m_setEditPoints.insert(pClone);
+	if(m_bOwnedByView)
+		m_pView->m_setEditPoints.insert(pClone);
 	return pClone;
 }
 
@@ -105,18 +105,44 @@ CEditPoint* CEditPoint::Clone() const throw(EViewIsAlreadyInavailable) {
 void CEditPoint::Convert(long cChars, RangeConvertType rct) {
 	AssertValid();
 	VERIFY_VIEW();
-	// TODO: CEditPoint::Convert()
+
+	if(m_pView->GetDocument()->IsReadOnly())
+		return;
+
+	CEditPoint	pt(m_pView);
+
+	pt.m_iLine = m_iLine;
+	pt.m_iChar = m_iChar;
+
+	if(cChars > 0)
+		pt.CharNext(cChars);
+	else {	// 폜̏ꍇ͌ȕ폜
+		while(cChars++ != 0) {
+			if(pt.m_iChar == 0) {
+				if(pt.m_iLine == 0)
+					break;
+				--pt.m_iLine;
+				pt.m_iChar = m_pView->GetDocument()->GetLineLength(pt.m_iLine);
+			} else
+				--pt.m_iChar;
+		}
+	}
+	Convert(pt, rct);
 }
 
 /**
  *	wʒu܂ł̃eLXgϊ
- *	@param pt	1̈ʒu
- *	@param rct	ϊ̎
+ *	@param pos	1̈ʒu
+ *	@param rct	ϊ̎ށBw͕s
  */
-void CEditPoint::Convert(const CEditPoint& pt, RangeConvertType rct) {
+void CEditPoint::Convert(const CCharPos& pos, RangeConvertType rct) {
 	AssertValid();
 	VERIFY_VIEW();
-	// TODO: CEditPoint::Convert()
+
+	if(m_pView->GetDocument()->IsReadOnly() || *this == pos)
+		return;
+
+//	if(rct == RCT)
 }
 
 /**
@@ -131,17 +157,17 @@ void CEditPoint::Copy(long cChars) throw(EViewIsAlreadyInavailable) {
 
 /**
  *	wʒu܂ł̃eLXgNbv{[hɃRs[
- *	@param cChars	1̈ʒu܂ł̕
+ *	@param pos	1̈ʒu
  */
-void CEditPoint::Copy(const CEditPoint& pt) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::Copy(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
-	CClipboard::SetClipboardText(m_pView->m_hWnd, GetText(pt).c_str());
+	CClipboard::SetClipboardText(m_pView->m_hWnd, GetText(pos).c_str());
 }
 
 /**
  *	wʒu܂ł̃eLXg폜ăNbv{[hɃRs[
- *	@param cChars	Rs[镶 (ł悢)
+ *	@param cChars	폜镶 (ł悢)
  */
 void CEditPoint::Cut(long cChars) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
@@ -155,16 +181,16 @@ void CEditPoint::Cut(long cChars) throw(EViewIsAlreadyInavailable) {
 
 /**
  *	wʒu܂ł̃eLXg폜ăNbv{[hɃRs[
- *	@param cChars	1̈ʒu܂ł̕ (ł悢)
+ *	@param pos	1̈ʒu
  */
-void CEditPoint::Cut(const CEditPoint& pt) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::Cut(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
 	if(m_pView->GetDocument()->IsReadOnly())
 		return;
-	CClipboard::SetClipboardText(m_pView->m_hWnd, GetText(pt).c_str());
-	Delete(pt);
+	CClipboard::SetClipboardText(m_pView->m_hWnd, GetText(pos).c_str());
+	Delete(pos);
 }
 
 /**
@@ -185,8 +211,8 @@ void CEditPoint::Delete(long cChars /* = 1 */) throw(EViewIsAlreadyInavailable)
 
 	if(cChars > 0)
 		pt.CharNext(cChars);
-	else {
-		while(cChars != 0) {
+	else {	// 폜̏ꍇ͌ȕ폜
+		while(cChars++ != 0) {
 			if(pt.m_iChar == 0) {
 				if(pt.m_iLine == 0)
 					break;
@@ -201,29 +227,63 @@ void CEditPoint::Delete(long cChars /* = 1 */) throw(EViewIsAlreadyInavailable)
 
 /**
  *	wʒu܂ł̃eLXg폜
- *	@param pt	1̈ʒu
+ *	@param pos	1̈ʒu
  */
-void CEditPoint::Delete(const CEditPoint& pt) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::Delete(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
 	if(m_pView->GetDocument()->IsReadOnly())
 		return;
-	*this = m_pView->GetDocument()->DeleteText(m_pView, *this, pt, OCF_PREVENTALL);
+	if(pos == *this)
+		return;
+	*this = m_pView->GetDocument()->DeleteText(this, *this, pos);
+	UpdateLastCx();
 }
 
-///	_r[̒ɗ悤ɃXN[
+///	_̍sr[̒ɗ悤ɃXN[
 void CEditPoint::EnsureCentered() throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
-	// TODO: CEditPoint::EnsureCentered()
+
+	const CCharPos		posDisplay = m_pView->DisplayCharFromLogicalChar(*this);
+	const unsigned int	cVisibleLines = m_pView->GetVisibleLineCount();
+
+	m_pView->OnHScroll(SB_THUMBPOSITION, 0, 0);
+	m_pView->OnVScroll(SB_THUMBPOSITION, posDisplay.m_iLine - cVisibleLines / 2, 0);
 }
 
 ///	_ɂȂ悤Ƀr[XN[
 void CEditPoint::EnsureVisible() throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
-	// TODO: CEditPoint::EnsureVisible()
+
+	const CCharPos		posDisplay = m_pView->DisplayCharFromLogicalChar(*this);
+	const unsigned int	cVisibleLines = m_pView->GetVisibleLineCount();
+	const unsigned int	cVisibleChars = m_pView->GetVisibleCharCount();
+
+	if(posDisplay.m_iLine <
+			m_pView->m_ptScroll.y * m_pView->m_layoutInfo.nVScrollRatio)	// ʂ
+		m_pView->OnVScroll(SB_THUMBPOSITION, posDisplay.m_iLine, 0);
+	else if(posDisplay.m_iLine -
+			m_pView->m_ptScroll.y *
+			m_pView->m_layoutInfo.nVScrollRatio > cVisibleLines - 1)	// ʂ艺
+		m_pView->OnVScroll(SB_THUMBPOSITION, posDisplay.m_iLine - cVisibleLines + 1, 0);
+	if(m_pView->m_ModeState.wpmWrapMode == WPM_NONE) {
+		const unsigned long	nScrollOffset =
+			m_pView->m_ptScroll.x * m_pView->m_layoutInfo.nHScrollRatio * m_pView->GetAvgCharWidth();
+		const unsigned long	cx =
+			m_pView->PosFromChar(*this).x + nScrollOffset - m_pView->m_layoutInfo.nLeftTabWidth + 1;
+		if(cx <= nScrollOffset)	// ʂ荶
+			m_pView->OnHScroll(SB_THUMBPOSITION,
+				cx / m_pView->GetAvgCharWidth() - cVisibleChars / 4, 0);
+		else if(cx > (m_pView->m_ptScroll.x * m_pView->m_layoutInfo.nHScrollRatio
+				+ cVisibleChars) * m_pView->GetAvgCharWidth())	// ʂE
+			m_pView->OnHScroll(SB_THUMBPOSITION,
+				cx / m_pView->GetAvgCharWidth() - cVisibleChars * 3 / 4, 0);
+	} else {
+		// ...
+	}
 }
 
 ///	hLg̐擪̕Ԃ
@@ -296,19 +356,19 @@ wstring CEditPoint::GetText(long cChars) const throw(EViewIsAlreadyInavailable)
 
 /**
  *	͈͓̃eLXgԂ
- *	@param pt	1̈ʒu
+ *	@param pos	1̈ʒu
  */
-wstring CEditPoint::GetText(const CEditPoint& pt) const throw(EViewIsAlreadyInavailable) {
+wstring CEditPoint::GetText(const CCharPos& pos) const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
 	Normalize();
-	if(*this == pt)
+	if(*this == pos)
 		return L"";
 
-	const CEditDoc*		pDoc = m_pView->GetDocument();
-	const CEditPoint&	posStart = std::min(*this, pt);
-	const CEditPoint&	posEnd = std::max(*this, pt);
+	const CEditDoc*	pDoc = m_pView->GetDocument();
+	const CCharPos&	posStart = std::min<CCharPos>(*this, pos);
+	const CCharPos&	posEnd = std::max<CCharPos>(*this, pos);
 
 	if(posStart.m_iLine == posEnd.m_iLine)	// 1s̏ꍇ
 		return pDoc->GetLine(posEnd.m_iLine).substr(
@@ -333,22 +393,182 @@ wstring CEditPoint::GetText(const CEditPoint& pt) const throw(EViewIsAlreadyInav
 }
 
 /**
- *	_̈ʒuɕ}B݈ʒu͓͕i߂
+ *	_̈ʒũeLXg폜ĕ}B݈ʒu͓͕i߂
  *	@param strText	}eLXg
  *	@param ocf		쌋tO
  */
-void CEditPoint::Insert(const wstring& strText,
-		Ascension::OperationConcatenationFlag ocf) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::DestructiveInsert(const wstring& strText) throw(EViewIsAlreadyInavailable) {
+	AssertValid();
+	VERIFY_VIEW();
+
+	CEditDoc*	pDoc = m_pView->GetDocument();
+
+	if(pDoc->IsReadOnly())
+		return;
+
+	CEditPoint	pos(m_pView);
+
+	pos.m_iLine = m_iLine;
+	pos.m_iChar = m_iChar;
+	pos.CharNext();
+	pDoc->EndEditCollection();
+	pDoc->BeginEditCollection();
+	pDoc->DeleteText(this, *this, pos);
+	*this = pDoc->InsertText(this, *this, strText);
+	pDoc->EndEditCollection();
+	UpdateLastCx();
+}
+
+/**
+ *	͈͓̃eLXgCfg
+ *	@param pos		1̈ʒu
+ *	@param chIndent	CfgɎg
+ *	@param nLevel	Cfgx
+ */
+CCharPos CEditPoint::Indent(const CCharPos& pos, wchar_t chIndent, long nLevel) {
+	AssertValid();
+	VERIFY_VIEW();
+
+	if(m_pView->GetDocument()->IsReadOnly())
+		return pos;
+
+	CEditDoc*		pDocument = m_pView->GetDocument();
+	const wstring	strIndent = wstring(abs(nLevel), chIndent);
+	const CCharPos	posTop = m_pView->GetSelTopPoint();
+	const CCharPos	posBottom = m_pView->GetSelBottomPoint();
+	CCharPos		posOtherRes = pos;
+	unsigned long	iLine = posTop.m_iLine;
+	unsigned long	cchIndent;	// s̋󔒕
+
+	if(iLine < posBottom.m_iLine && nLevel != 0) {
+		if(nLevel > 0) {
+			pDocument->InsertText(this, CCharPos(iLine, 0), strIndent);
+			if(iLine == posOtherRes.m_iLine && posOtherRes.m_iChar != 0)
+				posOtherRes.m_iChar += nLevel;
+			else if(iLine == m_iLine && m_iChar != 0)
+				m_iChar += nLevel;
+		} else {
+			cchIndent = CEditView::IsWhiteSpace(
+							pDocument->GetLine(iLine).c_str(),
+							pDocument->GetLineLength(iLine));
+			if(cchIndent > 0) {
+				unsigned long	cchDelete = min<unsigned long>(-nLevel, cchIndent);
+				pDocument->DeleteText(this, CCharPos(iLine, 0), CCharPos(iLine, cchDelete));
+				if(iLine == posOtherRes.m_iLine && posOtherRes.m_iChar != 0)
+					posOtherRes.m_iChar -= cchDelete;
+				else if(iLine == m_iLine && m_iChar != 0)
+					m_iChar -= cchDelete;
+			}
+		}
+	} else {	// I1sȓ -> Pȕ}
+		if(nLevel > 0)
+			m_pView->ReplaceSel(strIndent);
+		return *this;
+	}
+
+	// ÎSĂ̍s (t) Cfg
+	if(nLevel > 0) {
+		for(++iLine; iLine <= posBottom.m_iLine; ++iLine) {
+			if(pDocument->GetLineLength(iLine) != 0 && (iLine != posBottom.m_iLine || posBottom.m_iChar > 0)) {
+				pDocument->InsertText(this, CCharPos(iLine, 0), strIndent);
+				if(iLine == posOtherRes.m_iLine && posOtherRes.m_iChar != 0)
+					posOtherRes.m_iChar += nLevel;
+				else if(iLine == m_iLine && m_iChar != 0)
+					m_iChar += nLevel;
+			}
+		}
+	} else {
+		for(++iLine; iLine <= posBottom.m_iLine; ++iLine) {
+			cchIndent = CEditView::IsWhiteSpace(
+							pDocument->GetLine(iLine).c_str(),
+							pDocument->GetLineLength(iLine));
+			if(cchIndent > 0) {
+				unsigned long	cchDelete = min<unsigned long>(-nLevel, cchIndent);
+				pDocument->DeleteText(this, CCharPos(iLine, 0), CCharPos(iLine, cchDelete));
+				if(iLine == posOtherRes.m_iLine && posOtherRes.m_iChar != 0)
+					posOtherRes.m_iChar -= cchDelete;
+				else if(iLine == m_iLine && m_iChar != 0)
+					m_iChar -= cchDelete;
+			}
+		}
+	}
+	return posOtherRes;
+}
+
+/**
+ *	_̈ʒuɕ}B݈ʒu͓͕i߂
+ *	@param strText	}eLXg
+ */
+void CEditPoint::Insert(const wstring& strText) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
 	if(m_pView->GetDocument()->IsReadOnly())
 		return;
-	*this = m_pView->GetDocument()->InsertText(m_pView, *this, strText, ocf);
+	*this = m_pView->GetDocument()->InsertText(this, *this, strText);
+	UpdateLastCx();
+}
+
+/**
+ *	_̈ʒuɕ`}B݈ʒu͓͕i߂
+ *	@param strText	}eLXg
+ */
+void CEditPoint::InsertBox(const wstring& strText) throw(EViewIsAlreadyInavailable) {
+	AssertValid();
+	VERIFY_VIEW();
+
+	if(m_pView->GetDocument()->IsReadOnly() || strText.empty())
+		return;
+
+	CEditDoc*			pDocument = m_pView->GetDocument();
+	const unsigned long	cLines = pDocument->GetLineCount();
+	const wstring		pwszBreaks = CEditDoc::GetBreakString(pDocument->GetBreakType());
+	list<size_t>		listLines;
+
+	// sʒũXg
+	size_t	iChar = 0, iLast = 0;
+	while(true) {
+		iChar = strText.find_first_of(CEditDoc::m_wszBreakChars, iLast);
+		if(iChar == wstring::npos)
+			break;
+		iLast = iChar + ((iChar < strText.length() - 1
+			&& strText[iChar] == L'\r' && strText[iChar == L'\n']) ? 2 : 1);
+		listLines.push_back(iLast);
+	}
+
+	if(*listLines.begin() != 0)
+		listLines.push_front(0);
+
+	unsigned long	iLine = m_iLine;	// }s
+	wstring			strLine;			// }sƂȂ镔
+	unsigned long	xFirstLineInsert;	// }擪s̑}ʒu (r[ x W)
+	for(list<size_t>::const_iterator it = listLines.begin(); it != listLines.end(); ++it) {
+		if(it == listLines.begin())	// ŏ̍s
+			xFirstLineInsert = m_pView->PosFromChar(*this).x - m_pView->m_layoutInfo.nLeftTabWidth;
+		if(++it == listLines.end())	// ŏIs
+			strLine = strText.substr(*(--it), wstring::npos);
+		else {
+			size_t	cchLine = *it - ((strText[*it - 1] == L'\n' && strText[*it - 2] == L'\r') ? 2 : 1);
+			cchLine -= *(--it);
+			strLine = strText.substr(*it, cchLine);
+		}
+		++it;
+		if(iLine >= cLines - 1 && it != listLines.end())	// }ʒuɍs݂Ȃꍇ͍쐬
+			strLine += pwszBreaks;
+		--it;
+
+		if(strLine.length() != 0) {
+			strLine = m_pView->CalculateSpacesReachingVirtualPoint(iLine, xFirstLineInsert) + strLine;	// }ʒu̒
+			*this = pDocument->InsertText(this, CCharPos(iLine,
+				m_pView->CharFromPixel(iLine, xFirstLineInsert,
+					!m_pView->m_ModeState.bCaretMarkByMouse)), strLine);
+		}
+		++iLine;
+	}
 }
 
 ///	݈ʒuhLg̏I[ł邩ǂԂ
-bool CEditPoint::IsAtEndOfDocument() const throw(EViewIsAlreadyInavailable) {
+bool CEditPoint::IsEndOfDocument() const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -361,7 +581,7 @@ bool CEditPoint::IsAtEndOfDocument() const throw(EViewIsAlreadyInavailable) {
 }
 
 ///	݈ʒusł邩ǂԂ
-bool CEditPoint::IsAtEndOfLine() const throw(EViewIsAlreadyInavailable) {
+bool CEditPoint::IsEndOfLine() const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 	Normalize();
@@ -369,7 +589,7 @@ bool CEditPoint::IsAtEndOfLine() const throw(EViewIsAlreadyInavailable) {
 }
 
 ///	݈ʒuhLg̐擪ł邩ǂԂ
-bool CEditPoint::IsAtStartOfDocument() const throw(EViewIsAlreadyInavailable) {
+bool CEditPoint::IsStartOfDocument() const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 	Normalize();
@@ -377,7 +597,7 @@ bool CEditPoint::IsAtStartOfDocument() const throw(EViewIsAlreadyInavailable) {
 }
 
 ///	݈ʒusł邩ǂԂ
-bool CEditPoint::IsAtStartOfLine() const throw(EViewIsAlreadyInavailable) {
+bool CEditPoint::IsStartOfLine() const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 	Normalize();
@@ -392,10 +612,8 @@ void CEditPoint::LineDown(unsigned long cLines /* = 1 */) throw(EViewIsAlreadyIn
 	AssertValid();
 	VERIFY_VIEW();
 
-	unsigned long	cLines_ = m_pView->GetDocument()->GetLineCount();
-
 	Normalize();
-	m_iLine = (m_iLine + cLines < cLines_) ? m_iLine + cLines_ : cLines - 1;
+	m_iLine = min(m_iLine + cLines, m_pView->GetDocument()->GetLineCount() - 1);
 	m_iChar = m_pView->CharFromPixel(m_iLine, m_cxLast, true);
 }
 
@@ -408,7 +626,7 @@ void CEditPoint::LineUp(unsigned long cLines /* = 1 */) throw(EViewIsAlreadyInav
 	VERIFY_VIEW();
 
 	Normalize();
-	m_iLine = (m_iLine < cLines) ? 0 : m_iLine - cLines;
+	m_iLine = (m_iLine > cLines) ? m_iLine - cLines : 0;
 	m_iChar = m_pView->CharFromPixel(m_iLine, m_cxLast, true);
 }
 
@@ -436,6 +654,7 @@ void CEditPoint::MoveToAbsoluteCharOffset(unsigned long nOffset) throw(EViewIsAl
 	}
 	m_iLine = cLines - 1;
 	m_iChar = itLines->GetLine().length();
+	UpdateLastCx();
 }
 
 ///	hLg̏I[Ɉړ
@@ -446,6 +665,7 @@ void CEditPoint::MoveToEndOfDocument() throw(EViewIsAlreadyInavailable) {
 	const CEditDoc*	pDoc = m_pView->GetDocument();
 	m_iLine = pDoc->GetLineCount() - 1;
 	m_iChar = pDoc->GetLineLength(pDoc->GetLineCount() - 1);
+	UpdateLastCx();
 }
 
 ///	sɈړ
@@ -455,6 +675,7 @@ void CEditPoint::MoveToEndOfLine() throw(EViewIsAlreadyInavailable) {
 
 	m_iLine = std::min(m_iLine, m_pView->GetDocument()->GetLineCount() - 1);
 	m_iChar = m_pView->GetDocument()->GetLineLength(m_iLine);
+	UpdateLastCx();
 }
 
 ///	̃ubN}[Ns̍sɈړ
@@ -470,6 +691,7 @@ void CEditPoint::MoveToNextBookmark() throw(EViewIsAlreadyInavailable) {
 		if(m_pView->m_pLineLayoutManager->GetLine(iLine)->IsBookmarked()) {
 			m_iLine = iLine;
 			m_iChar = 0;
+			UpdateLastCx();
 			return;
 		}
 	}
@@ -479,6 +701,7 @@ void CEditPoint::MoveToNextBookmark() throw(EViewIsAlreadyInavailable) {
 		if(m_pView->m_pLineLayoutManager->GetLine(iLine)->IsBookmarked()) {
 			m_iLine = iLine;
 			m_iChar = 0;
+			UpdateLastCx();
 			return;
 		}
 	}
@@ -495,6 +718,7 @@ void CEditPoint::MoveToPoint(const CCharPos& pos) throw(EViewIsAlreadyInavailabl
 	m_iLine = pos.m_iLine;
 	m_iChar = pos.m_iChar;
 	Normalize();
+	UpdateLastCx();
 }
 
 ///	ÕubN}[Ns̍sɈړ
@@ -510,6 +734,7 @@ void CEditPoint::MoveToPrevBookmark() throw(EViewIsAlreadyInavailable) {
 			m_iLine = iLine;
 			m_iChar = 0;
 			return;
+			UpdateLastCx();
 		}
 		if(--iLine == 0)
 			break;
@@ -521,6 +746,7 @@ void CEditPoint::MoveToPrevBookmark() throw(EViewIsAlreadyInavailable) {
 		if(m_pView->m_pLineLayoutManager->GetLine(iLine)->IsBookmarked()) {
 			m_iLine = iLine;
 			m_iChar = 0;
+			UpdateLastCx();
 			return;
 		}
 	}
@@ -532,15 +758,52 @@ void CEditPoint::MoveToStartOfDocument() throw(EViewIsAlreadyInavailable) {
 	VERIFY_VIEW();
 
 	m_iLine = m_iChar = 0;
+	UpdateLastCx();
 }
 
-///	sɈړ
-void CEditPoint::MoveToStartOfLine() throw(EViewIsAlreadyInavailable) {
+/**
+ *	sɈړ
+ *	@param bToFirstText	true: 󔒕łȂŏ̕ɈړAfalse: 0ڂɈړB
+ *			݈ʒu󔒂Ŗŏ̕ł΁A0ڂɈړ
+ */
+void CEditPoint::MoveToStartOfLine(bool bToFirstText) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
-	m_iLine = std::min(m_iLine, m_pView->GetDocument()->GetLineCount() - 1);
-	m_iChar = 0;
+	const CEditDoc*	pDocument = m_pView->GetDocument();
+
+	m_iLine = std::min(m_iLine, pDocument->GetLineCount() - 1);
+	if(!bToFirstText)
+		m_iChar = 0;
+	else {
+		const wstring&	strLine = pDocument->GetLine(m_iLine);
+		unsigned long	iFirstText = CEditView::IsWhiteSpace(strLine.c_str(), strLine.length());
+		m_iChar = (m_iChar != iFirstText) ? iFirstText : 0;
+	}
+	UpdateLastCx();
+}
+
+///	sBݒɂăX}[gCfgAubNCfgs
+void CEditPoint::NewLine() throw(EViewIsAlreadyInavailable) {
+	AssertValid();
+	VERIFY_VIEW();
+
+	CEditDoc*	pDoc = m_pView->GetDocument();
+	wstring		strBreak = CEditDoc::GetBreakString(pDoc->GetBreakType());
+	bool		bDoneSmartIndent = false;	// X}[gCfg
+
+	if(pDoc->IsReadOnly())
+		return;
+//	if(m_ModeState.aitAutoIndentType == AIT_SMART)
+//		bDoneSmartIndent = SmartIndent(L'\n');
+	if(!bDoneSmartIndent
+			&& m_pView->m_ModeState.aitAutoIndentType == AIT_BLOCK) {	// I[gCfg
+		const wstring&	strCurrentLine = pDoc->GetLine(m_iLine);
+		unsigned long	cch = CEditView::IsWhiteSpace(strCurrentLine.c_str(), strCurrentLine.length());
+		strBreak += strCurrentLine.substr(0, cch);
+	}
+	if(!bDoneSmartIndent)
+		Insert(strBreak);
 }
 
 ///	ʒuɈړ (const \bh悭Ăяô const ɂĂ)
@@ -554,6 +817,76 @@ void CEditPoint::Normalize() const throw(EViewIsAlreadyInavailable) {
 		std::min(m_iChar, m_pView->GetDocument()->GetLineLength(m_iLine));
 }
 
+/**
+ *	hLgύXꂽƂɌĂяo
+ *	@param udi	XV̓e
+ */
+void CEditPoint::OnUpdateDocument(const TUpdateInfo& udi) {
+	AssertValid();
+	if(m_pView == 0)
+		return;
+
+//	Normalize();
+
+	// hLg̕ύXɍ킹Ĉʒu𒲐
+	if(udi.usSummary == US_OPERATION_INSERT) {	// }̏ꍇ
+		if(*this < udi.posBegin)	// ݈ʒu
+			return;
+		else if(this->m_iLine > udi.posBegin.m_iLine)	// ݍsO
+			m_iLine += udi.posResult.m_iLine - udi.posBegin.m_iLine;
+		else {	// ݍsƓs
+			m_iLine += udi.posResult.m_iLine - udi.posBegin.m_iLine;
+			m_iChar += udi.posResult.m_iChar - udi.posBegin.m_iChar;
+		}
+	} else if(udi.usSummary == US_OPERATION_DELETE) {	// 폜̏ꍇ
+		if(*this < udi.posEnd)	// ݈ʒu
+			return;
+		else if(this->m_iLine > udi.posEnd.m_iLine)	// ݍsO
+			m_iLine -= udi.posEnd.m_iLine - udi.posBegin.m_iLine;
+		else if(*this >= udi.posBegin && *this <= udi.posEnd) {	// ͈͓
+			m_iLine = udi.posBegin.m_iLine;
+			m_iChar = udi.posBegin.m_iChar;
+		} else {	// I_ݍsƓs
+			if(this->m_iLine == udi.posBegin.m_iLine)	// ͈͂1s
+				m_iChar -= udi.posEnd.m_iChar - udi.posBegin.m_iChar;
+			else {	// ͈͂s
+				m_iLine -= udi.posEnd.m_iLine - udi.posBegin.m_iLine;
+				m_iChar -= udi.posEnd.m_iChar - udi.posBegin.m_iChar;
+			}
+		}
+	} else
+		return;
+	UpdateLastCx();
+}
+
+/**
+ *	̃y[WɈړ
+ *	@param cPages	ړy[W
+ */
+void CEditPoint::PageDown(unsigned long cPages /* = 1 */) throw(EViewIsAlreadyInavailable) {
+	AssertValid();
+	VERIFY_VIEW();
+
+	Normalize();
+	m_iLine = min(m_pView->GetDocument()->GetLineCount() - 1,
+				m_iLine + (m_pView->GetVisibleLineCount() - 1) * cPages);
+	m_iChar = m_pView->CharFromPixel(m_iLine, m_cxLast, true);
+}
+
+/**
+ *	Õy[WɈړ
+ *	@param cPages	ړy[W
+ */
+void CEditPoint::PageUp(unsigned long cPages /* = 1 */) throw(EViewIsAlreadyInavailable) {
+	AssertValid();
+	VERIFY_VIEW();
+
+	Normalize();
+	m_iLine = (m_iLine > (m_pView->GetVisibleLineCount() - 1) * cPages) ?
+		m_iLine - (m_pView->GetVisibleLineCount() - 1) * cPages : 0;
+	m_iChar = m_pView->CharFromPixel(m_iLine, m_cxLast, true);
+}
+
 /**
  *	͈͓̃eLXgNbv{[h̓eŒu
  *	@param cChars	1̈ʒu܂ł̕ (ł悢)
@@ -561,61 +894,94 @@ void CEditPoint::Normalize() const throw(EViewIsAlreadyInavailable) {
 void CEditPoint::Paste(long cChars /* = 0 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
-	// TODO: CEditPoint::Paste()
+
+	if(m_pView->GetDocument()->IsReadOnly() || cChars == 0) {
+		Paste(*this);
+		return;
+	}
+
+	CEditPoint	pt(m_pView);
+
+	pt.m_iLine = m_iLine;
+	pt.m_iChar = m_iChar;
+
+	if(cChars > 0)
+		pt.CharNext(cChars);
+	else {	// 폜̏ꍇ͌ȕ폜
+		while(cChars++ != 0) {
+			if(pt.m_iChar == 0) {
+				if(pt.m_iLine == 0)
+					break;
+				--pt.m_iLine;
+				pt.m_iChar = m_pView->GetDocument()->GetLineLength(pt.m_iLine);
+			} else
+				--pt.m_iChar;
+		}
+	}
+	Paste(pt);
 }
 
 /**
  *	͈͓̃eLXgNbv{[h̓eŒu
- *	@param pt	1̈ʒu
+ *	@param pos	1̈ʒu
  */
-void CEditPoint::Paste(const CEditPoint& pt) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::Paste(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
-	// TODO: CEditPoint::Paste()
+
+	if(m_pView->GetDocument()->IsReadOnly())
+		return;
+
+	size_t		cch;
+	wchar_t*	pwszText = 0;
+	const UINT	nAvailableClipFormat = m_pView->CanPaste();
+
+	if(nAvailableClipFormat == 0)
+		return;
+	if(pos != *this)
+		Delete(pos);
+	cch = (CClipboard::GetClipboardTextSize(m_pView->m_hWnd) - 1);
+	pwszText = new wchar_t[cch + 1];
+	CClipboard::ReadClipboardText(m_pView->m_hWnd, pwszText, cch + 1);
+
+	if(nAvailableClipFormat == ::RegisterClipboardFormatW(L"MSDEVColumnSelect"))
+		InsertBox(pwszText);
+	else
+		Insert(pwszText);
+
+	delete[] pwszText;
 }
 
 /**
  *	͈͓̃eLXgXy[XCfg
- *	@param pt		1̈ʒu
+ *	@param pos		1̈ʒu
  *	@param nLevel	Cfgx
+ *	@return			Cfg <var>pos</var> ړׂʒu
  */
-void CEditPoint::SpaceIndent(const CEditPoint& pt, long nLevel /* = 1 */) throw(EViewIsAlreadyInavailable) {
+CCharPos CEditPoint::SpaceIndent(const CCharPos& pos, long nLevel /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
-	// TODO: CEditPoint::SpaceIndent()
-}
-
-/**
- *	͈͓̃eLXg̃Xy[X^uɕϊ
- *	@param cChars		1̈ʒu܂ł̕ (ł悢)
- *	@param bUntabify	^uXy[Xɕϊꍇ true
- */
-void CEditPoint::Tabify(long cChars, bool bUntabify /* = false */) throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	VERIFY_VIEW();
-	// TODO: CEditPoint::Tabify()
-}
-
-/**
- *	͈͓̃eLXg̃Xy[X^uɕϊ
- *	@param pt	1̈ʒu
- *	@param bUntabify	^uXy[Xɕϊꍇ true
- */
-void CEditPoint::Tabify(const CEditPoint& pt, bool bUntabify /* = false */) throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	VERIFY_VIEW();
-	// TODO: CEditPoint::Tabify()
+	return Indent(pos, L' ', nLevel);
 }
 
 /**
  *	͈͓̃eLXg^uCfg
- *	@param pt		1̈ʒu
+ *	@param pos		1̈ʒu
  *	@param nLevel	Cfgx
+ *	@return			Cfg <var>pos</var> ړׂʒu
  */
-void CEditPoint::TabIndent(const CEditPoint& pt, long nLevel /* = 1 */) throw(EViewIsAlreadyInavailable) {
+CCharPos CEditPoint::TabIndent(const CCharPos& pos, long nLevel /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
-	// TODO: CEditPoint::TabIndent()
+	return Indent(pos, L'\t', nLevel);
+}
+
+///	m_cxLast ݈ʒuɍXV
+void CEditPoint::UpdateLastCx() {
+	AssertValid();
+	m_cxLast = m_pView->PosFromChar(*this).x
+				+ m_pView->m_ptScroll.x * m_pView->m_layoutInfo.nHScrollRatio * m_pView->GetAvgCharWidth()
+				- m_pView->m_layoutInfo.nLeftTabWidth + 1;
 }
 
 /**
@@ -626,8 +992,11 @@ void CEditPoint::WordEndNext(unsigned long cWords /* = 1 */) throw(EViewIsAlread
 	AssertValid();
 	VERIFY_VIEW();
 
+	unsigned long	cLines = m_pView->GetDocument()->GetLineCount();
+
 	Normalize();
-	// TODO: CEditPoint::WordEndNext()
+	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, true, true);
+	UpdateLastCx();
 }
 
 /**
@@ -639,7 +1008,8 @@ void CEditPoint::WordEndPrev(unsigned long cWords /* = 1 */) throw(EViewIsAlread
 	VERIFY_VIEW();
 
 	Normalize();
-	// TODO: CEditPoint::WordEndPrev()
+	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, false, true);
+	UpdateLastCx();
 }
 
 /**
@@ -650,8 +1020,11 @@ void CEditPoint::WordNext(unsigned long cWords /* = 1 */) throw(EViewIsAlreadyIn
 	AssertValid();
 	VERIFY_VIEW();
 
+	unsigned long	cLines = m_pView->GetDocument()->GetLineCount();
+
 	Normalize();
-	// TODO: CEditPoint::WordNext()
+	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, true, false);
+	UpdateLastCx();
 }
 
 /**
@@ -663,29 +1036,63 @@ void CEditPoint::WordPrev(unsigned long cWords /* = 1 */) throw(EViewIsAlreadyIn
 	VERIFY_VIEW();
 
 	Normalize();
-	// TODO: CEditPoint::WordPrev()
+	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, false, false);
+	UpdateLastCx();
+}
+
+
+// CVisibleEditPoint class implementation
+/////////////////////////////////////////////////////////////////////////////
+
+/**
+ *	RXgN^
+ *	@param pView	Ώۂ̃r[
+ */
+CVisibleEditPoint::CVisibleEditPoint(CEditView* pView) : CEditPoint(pView) {
+}
+
+/**
+ *	Rs[RXgN^
+ *	@param rhs	E
+ */
+CVisibleEditPoint::CVisibleEditPoint(
+		const CVisibleEditPoint& rhs) : CEditPoint(rhs.m_pView) {
+	m_iLine = rhs.m_iLine;
+	m_iChar = rhs.m_iChar;
+	m_bSyncWithOtherEdit = rhs.m_bSyncWithOtherEdit;
+	m_cxLast = rhs.m_cxLast;
+}
+
+///	fXgN^
+CVisibleEditPoint::~CVisibleEditPoint() {
+}
+
+///	CCharPos ւ̌^ϊZq
+CVisibleEditPoint::operator CCharPos() const {
+	AssertValid();
+	return CCharPos(m_iLine, m_iChar);
+}
+
+///	Zq
+CVisibleEditPoint& CVisibleEditPoint::operator =(const CCharPos& rhs) {
+	AssertValid();
+
+	m_iLine = rhs.m_iLine;
+	m_iChar = rhs.m_iChar;
+	return *this;
+}
+
+///	@see	CEditPoint::OnUpdateDocument
+void CVisibleEditPoint::OnUpdateDocument(const TUpdateInfo& udi) {
+	AssertValid();
+
+	CEditPoint::OnUpdateDocument(udi);
 }
 
 
 // CEditRange class implementation
 /////////////////////////////////////////////////////////////////////////////
 #if 0
-///	RXgN^
-CEditRange::CEditRange(CEditView* pView, const CCharPos& posStart, const CCharPos& posEnd)
-		: m_pView(pView), m_pAnchorPoint(new CEditPoint(pView, posStart)), m_pActivePoint(new CEditPoint(pView, posEnd)) {
-}
-
-///	͈͓̃eLXg폜B̃\bh͏I_n_̈ʒuɈړ
-void CEditRange::Delete() throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	if(!IsAvailable())
-		throw EViewIsAlreadyInavailable();
-	if(m_pView->GetDocument()->IsReadOnly())
-		return;
-
-	Normalize();
-	m_pView->GetDocument()->DeleteText(m_pView, *m_pAnchorPoint, *m_pActivePoint, OCF_PREVENTALL);
-}
 
 ///	͈͓eLXgLs^CY
 void CEditRange::MakeCapitalized() throw(EViewIsAlreadyInavailable) {
@@ -716,119 +1123,6 @@ void CEditRange::MakeCapitalized() throw(EViewIsAlreadyInavailable) {
 	Replace(str);
 	delete[] pwsz;
 }
-
-///	͈͓eLXg
-void CEditRange::MakeLower() throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	if(!IsAvailable())
-		throw EViewIsAlreadyInavailable();
-	if(m_pView->GetDocument()->IsReadOnly())
-		return;
-
-	wstring		str = GetText();
-	wchar_t*	pwsz = new wchar_t[str.length()];
-
-	::CharLowerBuffW(pwsz, str.length());
-	str.assign(pwsz, str.length());
-	Replace(str);
-	delete[] pwsz;
-}
-
-///	͈͓eLXg啶
-void CEditRange::MakeUpper() throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	if(!IsAvailable())
-		throw EViewIsAlreadyInavailable();
-	if(m_pView->GetDocument()->IsReadOnly())
-		return;
-
-	wstring		str = GetText();
-	wchar_t*	pwsz = new wchar_t[str.length()];
-
-	::CharUpperBuffW(pwsz, str.length());
-	str.assign(pwsz, str.length());
-	Replace(str);
-	delete[] pwsz;
-}
-
-/**
- *	Xy[XCfg
- *	@param bReverse	tCfg̏ꍇ true
- */
-void CEditRange::SpaceIndent(bool bReverse /* = false */) throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	if(!IsAvailable())
-		throw EViewIsAlreadyInavailable();
-	if(m_pView->GetDocument()->IsReadOnly())
-		return;
-}
-
-/**
- *	󔒁A^u̕ϊ
- *	@param bRevoke	^u󔒂ɕϊƂ true
- */
-void CEditRange::Tabify(bool bRevoke /* = false */) throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	if(!IsAvailable())
-		throw EViewIsAlreadyInavailable();
-	if(m_pView->GetDocument()->IsReadOnly())
-		return;
-}
-
-/**
- *	^uCfg
- *	@param bReverse	tCfg̏ꍇ true
- */
-void CEditRange::TabIndent(bool bReverse /* = false */) throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	if(!IsAvailable())
-		throw EViewIsAlreadyInavailable();
-	if(m_pView->GetDocument()->IsReadOnly())
-		return;
-}
-
-///	͈͓eLXgNbv{[h̓eŒuB`\t͕s
-///	( CF_UNICODETEXT g)B̃\bh͏I_ũeLXg̖Ɉړ
-void CEditRange::Paste() throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	if(!IsAvailable())
-		throw EViewIsAlreadyInavailable();
-	if(m_pView->GetDocument()->IsReadOnly())
-		return;
-	if(!toBoolean(::IsClipboardFormatAvailable(CF_UNICODETEXT)))
-		return;
-
-	size_t		cch = CClipboard::GetClipboardTextSize(m_pView->m_hWnd);
-	wchar_t*	pwszText = new wchar_t[cch];
-
-	CClipboard::ReadClipboardText(m_pView->m_hWnd, pwszText, cch);
-	Replace(pwszText);
-	delete[] pwszText;
-}
-
-/**
- *	͈͓eLXguB̃\bh͏I_ũeLXg̖Ɉړ
- *	@param strText	ueLXg
- */
-void CEditRange::Replace(const wstring& strText) throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	if(!IsAvailable())
-		throw EViewIsAlreadyInavailable();
-
-	CEditDoc*	pDoc = m_pView->GetDocument();
-	CCharPos	posResult;
-
-	if(pDoc->IsReadOnly())
-		return;
-	if(IsEmpty())
-		posResult = pDoc->InsertText(m_pView, *m_pActivePoint, strText, OCF_PREVENTALL);
-	else {
-		pDoc->DeleteText(m_pView, *m_pAnchorPoint, *m_pActivePoint, OCF_PREVENTCONCATFORWARD);
-		posResult = pDoc->InsertText(m_pView, *m_pActivePoint, strText, OCF_PREVENTCONCATBACKWARD);
-	}
-	m_pActivePoint->m_iLine = posResult.m_iLine;
-	m_pActivePoint->m_iChar = posResult.m_iChar;
-}
 #endif
 
 /* [EOF] */
\ No newline at end of file
