@@ -1,5 +1,5 @@
 // EditView.h
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #ifndef _EDIT_VIEW_H_
 #define _EDIT_VIEW_H_
@@ -7,13 +7,14 @@
 #include "EditDoc.h"
 #include "..\..\Manah\DC.h"			// Manah::Windows::GDI::CMinimalDC
 #include "..\..\Manah\MenuOperator.h"
-#include "..\..\Manah\TextConverter.h"
+#include "..\..\Manah\Text.h"
 #include "..\..\Armaiti\DraggingTextDataObject.h"
 #include "LineLayout.h"
 #include "TextSearcher.h"
 #include "KeyMacroPlayer.h"
 #include "AutoCompleteWnd.h"
 #include <set>
+#include <algorithm>
 
 
 namespace Ascension {
@@ -23,6 +24,7 @@ class CEditController;
 class CLineLayoutInfo;
 class CLineLayoutManager;
 class CEditPoint;
+class CVisibleEditPoint;
 class CKeywordManager;
 class CKeyMacroPlayer;
 class CAutoCompleteWnd;
@@ -115,7 +117,7 @@ enum CommandIdentifier {
 	CMDID_EDIT_BACKSPACE,
 	///	݂̉sR[hŉs
 	CMDID_EDIT_BREAK,
-	/// ̓ ( : wchar_t)
+	/// ̓ (̃R[h|Cg : unsigned long)
 	CMDID_EDIT_CHAR,
 	///	Rs[
 	CMDID_EDIT_COPY,
@@ -125,10 +127,10 @@ enum CommandIdentifier {
 	CMDID_EDIT_DELETE,
 	///	ݍs폜
 	CMDID_EDIT_DELETELINE,
-	///	P̐擪܂ō폜
-	CMDID_EDIT_DELETETOWORDSTART,
-	///	P̖܂ō폜
-	CMDID_EDIT_DELETETOWORDEND,
+	///	O̒P̐擪܂ō폜
+	CMDID_EDIT_DELETETONEXTWORD,
+	///	̒P̐擪܂ō폜
+	CMDID_EDIT_DELETETOPREVWORD,
 	///	1s}
 	CMDID_EDIT_INSERTPREVLINE,
 	///	I͈͂Ls^CY
@@ -147,15 +149,15 @@ enum CommandIdentifier {
 	CMDID_EDIT_REDO,
 	/// ͕̓ ###  (ϊ̎ : NextCharConvert)
 	CMDID_EDIT_SETNEXTINPUTCONVERT,
-	/// Xy[XCfg
+	/// Xy[XCfg (Cfgx : unsigned short)
 	CMDID_EDIT_SPACEINDENT,
-	/// Xy[XtCfg
+	/// Xy[XtCfg (Cfgx : unsigned short)
 	CMDID_EDIT_SPACEUNINDENT,
 	///	I͈͂̋󔒂^uɕϊ
 	CMDID_EDIT_TABIFY,
-	/// ^uCfg
+	/// ^uCfg (Cfgx : unsigned short)
 	CMDID_EDIT_TABINDENT,
-	/// ^utCfg
+	/// ^utCfg (Cfgx : unsigned short)
 	CMDID_EDIT_TABUNINDENT,
 	///	̓ ( : wchar_t*)
 	CMDID_EDIT_TEXT,
@@ -176,22 +178,14 @@ enum CommandIdentifier {
 	CMDID_MOVE_BOOKMARKPREV,
 	///	I
 	CMDID_MOVE_CANCELSELECTION,
-	///	1Ɉړ
+	///	w蕶Ɉړ (ړ : unsigned long)
 	CMDID_MOVE_CHARNEXT,
-	///	1܂őI
+	///	w蕶܂őI (g : unsigned long)
 	CMDID_MOVE_CHARNEXTEXTEND,
-	///	1OɈړ
+	///	w蕶OɈړ (ړ : unsigned long)
 	CMDID_MOVE_CHARPREV,
-	///	1O܂őI
+	///	w蕶O܂őI (g : unsigned long)
 	CMDID_MOVE_CHARPREVEXTEND,
-	///	eLXg1sփXN[
-	CMDID_MOVE_DISPLAYLINEDOWN,
-	///	eLXg1sփXN[
-	CMDID_MOVE_DISPLAYLINEUP,
-	///	eLXg1y[WփXN[
-	CMDID_MOVE_DISPLAYNEXTPAGE,
-	///	eLXg1y[WփXN[
-	CMDID_MOVE_DISPLAYPREVPAGE,
 	///	Ɉړ
 	CMDID_MOVE_END,
 	///	܂őI
@@ -200,9 +194,9 @@ enum CommandIdentifier {
 	CMDID_MOVE_HOME,
 	///	܂őI
 	CMDID_MOVE_HOMEEXTEND,
-	/// 1sɈړ
+	/// wsɈړ (ړs : unsigned long)
 	CMDID_MOVE_LINEDOWN,
-	///	1s܂őI
+	///	ws܂őI (gs : unsigned long)
 	CMDID_MOVE_LINEDOWNEXTEND,
 	///	sɈړ
 	CMDID_MOVE_LINEEND,
@@ -212,42 +206,63 @@ enum CommandIdentifier {
 	CMDID_MOVE_LINEHOME,
 	///	s܂őI
 	CMDID_MOVE_LINEHOMEEXTEND,
-	/// 1sɈړ
+	/// wsɈړ (ړs : unsigned long)
 	CMDID_MOVE_LINEUP,
-	///	1s܂ňړ
+	///	ws܂őI (gs : unsigned long)
 	CMDID_MOVE_LINEUPEXTEND,
 	///	ΊʂɈړ
 	CMDID_MOVE_MATCHBRACKET,
 	///	Ίʂ܂ňړ
 	CMDID_MOVE_MATCHBRACKETEXTEND,
-	///	1y[WɈړ
+	///	wy[WɈړ (ړy[W : unsigned long)
 	CMDID_MOVE_PAGEDOWN,
-	///	1y[W܂őI
+	///	wy[W܂őI (gy[W : unsigned long)
 	CMDID_MOVE_PAGEDOWNEXTEND,
-	///	1y[WɈړ
+	///	wy[WɈړ (ړy[W : unsigned long)
 	CMDID_MOVE_PAGEUP,
-	///	1y[W܂őI
+	///	wy[W܂őI (gy[W : unsigned long)
 	CMDID_MOVE_PAGEUPEXTEND,
 	///	SđI
 	CMDID_MOVE_SELECTALL,
 	///	݂̒PI
 	CMDID_MOVE_SELECTCURRENTWORD,
-	///	̒P̏I[Ɉړ
+	///	̒P̏I[Ɉړ (ړꐔ : unsigned long)
 	CMDID_MOVE_WORDENDNEXT,
-	///	̒P̏I[܂őI
+	///	̒P̏I[܂őI (gꐔ : unsigned long)
 	CMDID_MOVE_WORDENDNEXTEXTEND,
-	///	O̒P̏I[Ɉړ
+	///	O̒P̏I[Ɉړ (ړꐔ : unsigned long)
 	CMDID_MOVE_WORDENDPREV,
-	///	O̒P̏I[܂őI
+	///	O̒P̏I[܂őI (gꐔ : unsigned long)
 	CMDID_MOVE_WORDENDPREVEXTEND,
-	///	̒P̐擪Ɉړ
+	///	̒P̐擪Ɉړ (ړꐔ : unsigned long)
 	CMDID_MOVE_WORDNEXT,
-	///	̒P̐擪܂őI
+	///	̒P̐擪܂őI (gꐔ : unsigned long)
 	CMDID_MOVE_WORDNEXTEXTEND,
-	///	O̒P̐擪Ɉړ
+	///	O̒P̐擪Ɉړ (ړꐔ : unsigned long)
 	CMDID_MOVE_WORDPREV,
-	///	O̒P̐擪܂őI
+	///	O̒P̐擪܂őI (gꐔ : unsigned long)
 	CMDID_MOVE_WORDPREVEXTEND,
+
+	///	1EփXN[
+	CMDID_SCROLL_COLUMNNEXT,
+	///	1񍶂փXN[
+	CMDID_SCROLL_COLUMNPREV,
+	///	ŏIsփXN[
+	CMDID_SCROLL_END,
+	///	Lbgɗ悤ɃXN[
+	CMDID_SCROLL_ENSURECARETCENTERED,
+	///	LbgɂȂ悤ɃXN[
+	CMDID_SCROLL_ENSURECARETVISIBLE,
+	///	擪sփXN[
+	CMDID_SCROLL_HOME,
+	///	1sփXN[
+	CMDID_SCROLL_LINEDOWN,
+	///	1sփXN[
+	CMDID_SCROLL_LINEUP,
+	///	1y[WփXN[
+	CMDID_SCROLL_PAGEDOWN,
+	///	1y[WփXN[
+	CMDID_SCROLL_PAGEUP,
 };
 
 ///	Cfg̎
@@ -260,16 +275,6 @@ enum AutoIndentType {
 	AIT_SMART
 };
 
-///	SetSel Q
-enum SelCorrectingDirection {
-	///	
-	SCD_INVARIANT,
-	///	
-	SCD_LEFT,
-	///	E
-	SCD_RIGHT
-};
-
 /// ɓ͂镶̕ϊ
 enum NextCharConvert {
 	/// ϊ
@@ -280,22 +285,30 @@ enum NextCharConvert {
 	NCC_ACUTE,
 	/// Ȑ܃ANZg
 	NCC_CIRCUMFLEX,
-	/// `_
+	/// @
 	NCC_TILDE,
 	/// EEg
 	NCC_DIAERESIS,
-	/// 
-	NCC_RINGABOVE,
 	/// ZfB[
 	NCC_CEDILLA,
-	/// ΐ
-	NCC_STROKE,
-	/// ss
-	NCC_SHARP,
-	/// eth
-	NCC_ETH,
+	///	t
+	NCC_SUPERSCRIPT,
+	///	t
+	NCC_SUBSCRIPT
+};
+
+/// ҏW̎
+enum EditOperationType {
 	/// 
-	NCC_COMBINED
+	EOT_NONE,
+	/// ^CsO
+	EOT_TYPING,
+	/// 폜
+	EOT_DELETE,
+	/// ㏑
+	EOT_REPLACE,
+	/// \t
+	EOT_PASTE,
 };
 
 ///	CmdEditConvertSel ̕ϊ[h
@@ -401,12 +414,14 @@ const EditViewOption	EVO_SHOW_WHITESPACE			= 0x0010;
 const EditViewOption	EVO_SHOW_UNICODECTRLCHARS	= 0x0020;
 ///	IɑĂs̔wih
 const EditViewOption	EVO_PAINTBREAKCONTINUINGSEL	= 0x0040;
-///	hbOAhhbvIێ
-const EditViewOption	EVO_KEEPSELONDRAGDROP		= 0x0080;
+///	tHggpɕԊu1
+const EditViewOption	EVO_MAKECLOSEBOLDCHARS		= 0x0080;
 ///	\P⊮ő啶ʂȂ
 const EditViewOption	EVO_IGNORECASE				= 0x0100;
 ///	L[{[hŃLbgړłȂ̑OɃ}EXňړł悤ɂ
 const EditViewOption	EVO_ACCEPTABLECARETBYMOUSE	= 0x0200;
+///	g[NPʂŃZbg
+const EditViewOption	EVO_RESETDIRECTIONBYTOKEN	= 0x0400;
 
 
 ///	CEditView ̐ݒ荀
@@ -594,6 +609,19 @@ private:
  *	L̂͊{Iɍŏ̃r[ANX GetPrimaryView
  *	\bhŊmɏLr[擾ł
  *
+ *	<h3>`̓Ɖ</h3>
+ *
+ *	܂Ƃ܂sۂɑSĂ̒iKŕ`sĂƓ삪xȂ̂ŁA
+ *	CEditView ͕`̓s߂ CEditView::Freeze \bhA
+ *	邽߂ CEditView::Unfreeze \bh2pӂĂB
+ *	ACEditView::InvalidateLine ACEditView::InvalidateLines
+ *	͑ɂ͍ĕ`s킸A`̕KvȍsL^ĂA
+ *	𓀂ƂɈxɕ`sB̂ CEditView ̂قƂǂ̑
+ *	CWindow::InvalidateRect 𒼐ڌĂяôł͂ȂAL2̃\bhōĕ`s
+ *
+ *	֐̂߂ɓƂ̉s\bh͎̕A
+ *	/s (J CEditView::_Unfreeze g)
+ *
  *	@see	CEditDoc, CEditController
  */
 class CEditView :
@@ -616,6 +644,7 @@ class CEditView :
 	friend class CEditController;
 	friend class CLineLayoutManager;
 	friend class CEditPoint;
+	friend class CBoundarySearcher;
 	friend class CAutoCompleteWnd;
 
 	// f[^o
@@ -624,6 +653,7 @@ private:
 	static CClipboardRing					m_oClipboardRing;	// Nbv{[hO
 	static CTextSearcher					m_oTextSearcher;	// eLXg
 	static CPopupMenu						m_oContextMenu;		// ftHg̃ReLXgj[
+	CBoundarySearcher*						m_pBoundarySearcher;// eLXgĚ
 	Armaiti::OLE::CDraggingTextDataObject*	m_pDragging;		// OLE hbOIuWFNg
 	CKeyMacroPlayer*						m_pKeyMacroPlayer;	// L[}N̊Ǘ
 	CAutoCompleteWnd*						m_pwndAutoComplete;	// ͕⊮EBhE
@@ -631,7 +661,7 @@ private:
 	std::set<CEditPoint*>					m_setEditPoints;	// ێĂҏW_
 
 	/* LÂ̊Ǘ (NX̐Q) */
-	CEditView*		m_pOriginalView;		//  (null łΎ)
+	CEditView*		m_pOriginalView;		//  (this łΎ)
 	std::set<CEditView*>	m_setClones;	// ̕
 
 	/* CAEg */
@@ -672,6 +702,8 @@ private:
 	/* [h */
 	struct TEditViewModeState {
 		bool			bOvertype;			// ㏑[h
+		bool			bRtlReading;		// E獶ɓǂނǂ
+		bool			bResetDirByToken;	// g[NƂɕZbg邩ǂ
 		bool			bShowUnicodeCtrls;	// Unicode 䕶\邩ǂ ()
 		bool			bBoxSelecting;		// `I ({^ĂȂĂ悢)
 		bool			bCaretMarkByMouse;	// Lbg󂯎Ȃ̑OɃ}EXňړł
@@ -682,7 +714,8 @@ private:
 		AutoIndentType	aitAutoIndentType;	// Cfg̎
 		unsigned long	cFindLimit;			// X}[gCfggpɉsO܂ł̊ʂɑΉ邩
 
-		TEditViewModeState() : bOvertype(false), bShowUnicodeCtrls(false), bBoxSelecting(false),
+		TEditViewModeState() : bOvertype(false), bRtlReading(false),
+			bResetDirByToken(true), bShowUnicodeCtrls(false), bBoxSelecting(false),
 			bCaretMarkByMouse(false), lttTabType(LTT_LINENUMBER), wpmWrapMode(WPM_NONE), nWrapWidth(100),
 			bUseOLEDragDrop(true), aitAutoIndentType(AIT_BLOCK), cFindLimit(100) {
 		}
@@ -699,15 +732,13 @@ private:
 	LeftDownMode	m_ldmLeftDown;				// LeftDownMode `Q
 	NextCharConvert	m_nccNextCharConvert;		// NextCharConvert `Q
 	TEditOperation	m_eoLast;					// OɍsҏW
-	bool			m_bIgnoreSelChange;			// OnUpdate Ō݈ʒu̕ύX𖳎B
-												// ̃tO؂ւ邱Ƃɂĕ`ʂ炷ƂłB
-												// XVeŜꎞIɖƑ̃r[ŉeo
-												// (͓̃r[ĂӖ) A
-												// ݈ʒu̓r[ŗL̂̂Ȃ̂ŖȂ
+	bool			m_bFreezed;					// t[Y
+	std::set<ulong>	m_setInvalidLines;			// t[Yɍĕ`vꂽsB
+												// InvalidateLine(s)  OnUpdate Ǘ
 
 	/* ݈ʒu */
-	CCharPos		m_posAnchor;		// IJnʒu
-	CCharPos		m_posActive;		// IIʒu (Lbgʒu)
+	CVisibleEditPoint*	m_pAnchorPoint;	// IJnʒu (AJ[|Cg)
+	CVisibleEditPoint*	m_pActivePoint;	// IIʒu (ANeBu|CgALbgʒu)
 	POINT			m_ptScroll;			// XN[o[̈ʒu (x:Ay:)B
 										// ̒l m_nHScrollRatio  m_nVScrollRatio 悸
 										// \Jnʒu (:ϕPʁA:\sP) 
@@ -715,10 +746,7 @@ private:
 										// _s<->\sϊ̂߂̃LbV
 										// cXN[ (OnVScroll)A܂Ԃ񂪕ωƂ (OnSize,
 										// ModifyFontInfo)A\Jn_sO̍sҏWꂽƂ
-										// (OnUpdate) XV
-	unsigned long	m_nLastPixel;		// Lbg𓮂n߂O̍[̈ʒu (r[W)B
-										// ̒l GetSel Ō݈ʒuɏ邪A
-										// []A[] L[ňړƂ̂ݑO̒lێ
+										// (OnUpdate) XV <- !݂͎gĂȂ!
 
 	/* \ */
 	typedef map<
@@ -731,6 +759,7 @@ private:
 		bool			bDrawSpaces;			// 󔒂̑֕`悷邩ǂ
 		bool			bIgnoreCaseOnHilite;	// \RgJn/I񌟍ɑ啶ʂȂ
 		bool			bPaintBreakContinuingSelection;	///	IɑĂs̔wih邩ǂ
+		bool			bDrawBoldCharsClosely;	// l߂ĕ`悷邩ǂ
 		wchar_t			chTab;					// ^ȗ֕
 		wchar_t			chIdeographicSpace;		// Sp󔒂̑֕
 		wchar_t			chSpace;				// ̑̋󔒗ޕ̑֕
@@ -816,12 +845,14 @@ public:
 	DWORD			GetLineParam(unsigned long iLine) const;						// AvP[V`l̎擾
 	void			GetOption(TEditViewOption& evo) const;							// GȐݒ̎擾
 	unsigned short	GetTabWidth() const;											// ^u̎擾
+	bool			HasRtlTextDirection() const;
 
 	// ̐ݒ
 	void	ModifyFontInfo(const LOGFONTW& lf, bool bRedraw = true);		// tHg̕ύX
 	void	SetAppDefinedLine(unsigned long iLine);							// AvP[V`s̐ݒ
 	void	SetCandidateWords(const std::list<TCandidateWord>& listWords);	// ͌PꃊXg̐ݒ
 	void	SetCharSpaces(unsigned int nLineSpan, unsigned int nCharSpan);	// sÅԊu̕ύX
+	void	SetTextDirection(bool bRtlReading);
 	void	SetEventListener(IEditViewEventListener* pEventListener);		// CxgXi̓o^
 	void	SetLeftTabType(LeftTabType ltt);								// [̑т̐ݒ
 	void	SetLineParam(unsigned long iLine,								// AvP[V`l̐ݒ
@@ -848,6 +879,8 @@ public:
 	void	UnlockParsing();												// s̗͂}
 
 	// Lbg
+	void		BeginBoxSelect();
+	void		EndBoxSelect();
 	CCharPos	GetActivePoint() const;
 	CCharPos	GetAnchorPoint() const;
 	void		GetSel(CCharPos& posAnchor, CCharPos& posActive) const;			// I͈͂̎擾
@@ -886,52 +919,52 @@ public:
 				bool bOn = true) throw(out_of_range);
 	void	ToggleBookmark(unsigned long iLine) throw(out_of_range);		// ubN}[N̐ݒ/
 
+	// `̓A
+	void	Freeze();
+	void	Unfreeze();
+
 	// R}h
 	void	ExecCommand(CommandIdentifier cmd, LPARAM lParam = 0L);	// R}ḧꊇ
 protected:
 	/* ړƑI */
 	void	CmdMoveCancelSelection();
-	void	CmdMoveCharNext(bool bSelect = false);
-	void	CmdMoveCharPrev(bool bSelect = false);
-	void	CmdMoveDisplayLineUp();
-	void	CmdMoveDisplayLineDown();
-	void	CmdMoveDisplayNextPage();
-	void	CmdMoveDisplayPrevPage();
-	void	CmdMoveEnd(bool bSelect = false);
-	void	CmdMoveHome(bool bSelect = false);
-	void	CmdMoveLineDown(bool bSelect = false);
-	void	CmdMoveLineEnd(bool bSelect = false);
-	void	CmdMoveLineHome(bool bSelect = false);
-	void	CmdMoveLineUp(bool bSelect = false);
-	void	CmdMoveMatchBracket(bool bSelect = false);
+	void	CmdMoveCharNext(bool bExtend = false, unsigned long cch = 1);
+	void	CmdMoveCharPrev(bool bExtend = false, unsigned long cch = 1);
+	void	CmdMoveEnd(bool bExtend = false);
+	void	CmdMoveHome(bool bExtend = false);
+	void	CmdMoveLineDown(bool bExtend = false, unsigned long cLines = 1);
+	void	CmdMoveLineEnd(bool bExtend = false);
+	void	CmdMoveLineHome(bool bExtend = false);
+	void	CmdMoveLineUp(bool bExtend = false, unsigned long cLines = 1);
+	void	CmdMoveMatchBracket(bool bExtend = false);
 	void	CmdMoveNextBookmark();
-	void	CmdMovePageDown(bool bSelect = false);
-	void	CmdMovePageUp(bool bSelect = false);
+	void	CmdMovePageDown(bool bExtend = false, unsigned long cPages = 1);
+	void	CmdMovePageUp(bool bExtend = false, unsigned long cPages = 1);
 	void	CmdMovePrevBookmark();
 	void	CmdMoveSelectAll();
 	void	CmdMoveSelectCurrentWord();
-	void	CmdMoveWordEndNext(bool bSelect = false);
-	void	CmdMoveWordEndPrev(bool bSelect = false);
-	void	CmdMoveWordNext(bool bSelect = false);
-	void	CmdMoveWordPrev(bool bSelect = false);
+	void	CmdMoveWordEndNext(bool bExtend = false, unsigned long cWords = 1);
+	void	CmdMoveWordEndPrev(bool bExtend = false, unsigned long cWords = 1);
+	void	CmdMoveWordNext(bool bExtend = false, unsigned long cWords = 1);
+	void	CmdMoveWordPrev(bool bExtend = false, unsigned long cWords = 1);
 	/* ҏW */
-	void	CmdEditChar(wchar_t ch);
+	void	CmdEditChar(unsigned long nCodePoint);
 	void	CmdEditBreak();
 	void	CmdEditInsertPrevLine();
 	void	CmdEditBackspace();
 	void	CmdEditDelete();
 	void	CmdEditDeleteLine();
-	void	CmdEditDeleteToWordEnd();
-	void	CmdEditDeleteToWordStart();
+	void	CmdEditDeleteToNextWord();
+	void	CmdEditDeleteToPrevWord();
 	void	CmdEditUndo();
 	void	CmdEditRedo();
 	void	CmdEditConvertSel(SelConvertType sct);
 	void	CmdEditCut();
 	void	CmdEditCopy();
 	void	CmdEditPaste();
-	void	CmdEditSpaceIndent(bool bReverse = false);
+	void	CmdEditSpaceIndent(bool bReverse = false, unsigned short nLevel = 1);
 	void	CmdEditTabify(bool bRevoke = false);
-	void	CmdEditTabIndent(bool bReverse = false);
+	void	CmdEditTabIndent(bool bReverse = false, unsigned short nLevel = 1);
 	void	CmdEditText(const wchar_t* pwszText);
 	void	CmdEditToggleCharCodePoint();
 	void	CmdEditToggleOvertypeMode();
@@ -939,6 +972,17 @@ protected:
 	void	CmdEditToggleIMEStatus();
 	void	CmdEditRecomposeSelection();
 	void	CmdEditOpenCandidateWindow();
+	/* XN[ */
+	void	CmdScrollColumnNext();
+	void	CmdScrollColumnPrev();
+	void	CmdScrollEnd();
+	void	CmdScrollEnsureCaretCentered();
+	void	CmdScrollEnsureCaretVisible();
+	void	CmdScrollHome();
+	void	CmdScrollLineDown();
+	void	CmdScrollLineUp();
+	void	CmdScrollPageDown();
+	void	CmdScrollPageUp();
 	/* L[}N */
 public:
 	void	CmdKeymacroPlay();									// L[}N̎s
@@ -960,8 +1004,8 @@ public:
 	unsigned long	GetDisplayLineCount() const;					// \sԂ
 	void			GetDisplayLineOffsetIndex(unsigned long iDisplayLine,		// \s_sƃItZbgɕϊ
 						unsigned long& iLogicalLine, unsigned long& iOffset) const;
-	unsigned int	GetVisibleLineCount() const;					// EBhEƕ̍\\ȍsvZ
-	unsigned int	GetVisibleCharCount() const;					// EBhEƕ̕\\1sɕvZ
+	int				GetVisibleLineCount() const;					// EBhEƕ̍\\ȍsvZ
+	int				GetVisibleCharCount() const;					// EBhEƕ̕\\1sɕvZ
 	bool			IsOverSelection(const POINT& pt) const;						// ʒuI͈͏ォǂ
 	bool			IsOverURIString(const POINT& pt, std::wstring& strURI) const;	// ʒu URI ォǂ
 	CCharPos		LogicalCharFromDisplayChar(const CCharPos& posDisplay) const;	// \ʒu_ʒuɕϊ
@@ -1013,10 +1057,10 @@ public:
 
 	// J[eBeB (I͈̓eLXgwp)
 protected:
-	void	DeleteSel(OperationConcatenationFlag ocf);									// I͈͂̍폜
-	void	InsertText(const std::wstring& strText, OperationConcatenationFlag ocf);	// Lbgʒuւ̑̕}
+	void	DeleteSel();															// I͈͂̍폜
+	void	InsertText(const std::wstring& strText);								// Lbgʒuւ̑̕}
 public:	// b...
-	void	ReplaceSel(const std::wstring& strText, OperationConcatenationFlag ocf, bool bBoxPaste = false);	// I͈͂̒u
+	void	ReplaceSel(const std::wstring& strTextbool, bool bBoxPaste = false);	// I͈͂̒u
 
 	// J[eBeB ()
 protected:
@@ -1036,19 +1080,20 @@ protected:
 
 	// J[eBeB ()
 protected:
-	static bool	IsCharAcceptableCaret(unsigned long nCodePoint);
 	static bool	IsIdentifierContinueChar(unsigned long nCodePoint);
 	static bool	IsIdentifierStartChar(unsigned long nCodePoint);
 				
 	// J[eBeB (`)
 protected:
-	virtual void			DrawBreakMark(int x, int y, BreakType bt, COLORREF clrBack);	// s}[N̕`
+	virtual int				DrawBreakMark(int x, int y, BreakType bt, COLORREF clrBack);	// s}[N̕`
 	virtual void			DrawLeftTab(unsigned long iStart, unsigned long iEnd);			// т`
 	virtual unsigned long	DrawLine(unsigned long iLine, int y,							// 1s`
 								const std::wstring& strLine, BreakType bt, const CLineLayoutInfo* pLineInfo);
 	HFONT					GetFontForRenderingToken(TextType type) const;					// w肵g[N`tHgԂ
 	void					InvalidateLine(unsigned long iLine);							// ws`𖳌
 	void					InvalidateLines(unsigned long iStart, unsigned long iEnd);		// wԍs`𖳌
+private:
+	void					_Unfreeze();
 
 	// J[eBeB (ʒuvZ)
 protected:
@@ -1061,23 +1106,29 @@ protected:
 	static int		MakeCaretPosValid(const wchar_t* pwsz,							// LbgʒȕC
 						unsigned long cch, unsigned long nCaretPos, bool bBackward);
 
+	// J[eBeB (I͈͂ƃLbg)
+protected:
+	void						CheckMatchBrackets();
+	const CVisibleEditPoint&	GetSelTopPoint() const;
+	const CVisibleEditPoint&	GetSelBottomPoint() const;
+	virtual void				OnMoveCaret();
+
 	// J[eBeB (̑)
 protected:
-	virtual int		CompareString(const wchar_t* psz1,
-						const wchar_t* psz2, unsigned long cch) const;				// ̔r
-	virtual bool	FindBracket(const CCharPos& pos, CCharPos& posFound) const;		// Ίʂ
-	unsigned int	GetLineNumberMaxDigit() const;									// s̍ő包Ԃ
-	void			InitializeWindow();
-	void			InsertBoxText(const CCharPos& pos,								// `̑}
-						const std::wstring& strText, OperationConcatenationFlag ocf);
-	bool			IsAcceptableAsMacroKey(UINT nChar) const;						// L[}NŋL^\ȃL[
-	static bool		IsNeutralNoAlpha(wchar_t wch);									// ԕ̒P\ǂ𒲂ׂ
-	void			ModifyScrollInfo(bool bHorizontal, bool bVertical);				// XN[o[ݒ̍XV
-	void			RecalcLeftTabWidth(int cy = -1);								// m_nLeftTabWidth XV
-	static bool		RegisterWindowClass();											// Ascension EBhENX̓o^
-	virtual void	UpdateGDIObjects();												// GDI IuWFNg̍XV
-	virtual void	ValidateCaretPos();												// Lbg𐳂ʒuɈړ
-	void			ValidateIMEWindowPos();											// IME EBhE𐳂ʒuɈړ
+	virtual int			CompareString(const wchar_t* psz1,
+							const wchar_t* psz2, unsigned long cch) const;
+	static wchar_t		ConvertCharacter(wchar_t wch, NextCharConvert ncc);
+	virtual bool		FindBracket(const CCharPos& pos, CCharPos& posFound) const;
+	unsigned int		GetLineNumberMaxDigit() const;
+	void				InitializeWindow();
+	bool				IsAcceptableAsMacroKey(UINT nChar) const;
+	static bool			IsNeutralNoAlpha(wchar_t wch);
+	void				ModifyScrollInfo(bool bHorizontal, bool bVertical);
+	void				RecalcLeftTabWidth(int cy = -1);
+	static bool			RegisterWindowClass();
+	virtual void		UpdateGDIObjects();
+	virtual void		ValidateCaretPos();
+	void				ValidateIMEWindowPos();
 
 	// J[eBeB (̑) ʂɌJ
 public:
@@ -1114,7 +1165,7 @@ protected:
 
 	// WbZ[Wnh
 	virtual void	OnDocumentSetup(LPARAM lHint = 0);								// hLgA[hꂽ
-	virtual void	OnUpdate(Manah::Windows::CView* pSender, LPARAM lParam, void* pHint);	// hLg̒ʐM
+	virtual void	OnUpdate(Manah::CObject* pSender, LPARAM lParam, void* pHint);	// hLg̒ʐM
 };
 
 // CC֐̎
@@ -1183,18 +1234,6 @@ inline bool CEditView::CharIsInCommentOrQuotation(const CCharPos& pos) const {
 	return false;
 }
 
-///	ANeBu|CgԂ
-inline CCharPos CEditView::GetActivePoint() const {
-	AssertValid();
-	return m_posActive;
-}
-
-///	AJ[|CgԂ
-inline CCharPos CEditView::GetAnchorPoint() const {
-	AssertValid();
-	return m_posAnchor;
-}
-
 /**
  *	ݎgpĂtHg (C^bNłȂ) 
  *	1`悷Ƃ̕ϕԂ
@@ -1287,7 +1326,7 @@ inline void CEditView::GetOption(TEditViewOption& evo) const {
 	wchar_t				wszSurrogates[3] = {0, 0, 0};
 	for(list<ulong>::const_iterator it = listAlphas.begin(); it != listAlphas.end(); ++it) {
 		if(*it >= 0x010000) {
-			Manah::CTextConverter::EncodeCodePointToUTF16SurrogatePair(*it, wszSurrogates);
+			Manah::Text::EncodeCodePointToUTF16SurrogatePair(*it, wszSurrogates);
 			ssChars << wszSurrogates;
 		} else
 			ssChars << static_cast<wchar_t>(*it);
@@ -1345,7 +1384,7 @@ inline CEditView* CEditView::GetViewKeyMacroRecording() {
  *	NCAg̈ɕ\ł񐔂擾
  *	@return	\\ȗ
  */
-inline unsigned int CEditView::GetVisibleCharCount() const {
+inline int CEditView::GetVisibleCharCount() const {
 	RECT	rect;
 	GetClientRect(&rect);
 	return (rect.right - rect.left == 0) ? 0 :
@@ -1356,7 +1395,7 @@ inline unsigned int CEditView::GetVisibleCharCount() const {
  *	NCAg̈ɕ\łs擾
  *	@return	\\ȍs
  */
-inline unsigned int CEditView::GetVisibleLineCount() const {
+inline int CEditView::GetVisibleLineCount() const {
 	RECT	rect;
 	GetClientRect(&rect);
 	return (rect.bottom - rect.left == 0) ? 0:
@@ -1369,13 +1408,10 @@ inline WrapMode CEditView::GetWrapMode() const {
 	return m_ModeState.wpmWrapMode;
 }
 
-/**
- *	I͈͂邩ǂԂ
- *	@return	I͈͂Ȃ true
- */
-inline bool CEditView::HasSelection() const {
+///	E獶ɓǂރ[h̏ꍇ true Ԃ
+inline bool CEditView::HasRtlTextDirection() const {
 	AssertValid();
-	return m_posAnchor != m_posActive;
+	return m_ModeState.bRtlReading;
 }
 
 /**
@@ -1431,15 +1467,19 @@ inline unsigned long CEditView::IsDoubleQuotation(const wchar_t* psz, unsigned l
 inline unsigned long CEditView::IsIdentifier(const wchar_t* psz, unsigned long cch) const {
 	assert(psz != 0);
 
-	unsigned long	nCodePoint;
+	using namespace Manah::Text;
+
+	const std::list<unsigned long>&	listAlphas = m_FoundationInfo.listCharsTreatedAsAlpha;
+	unsigned long					nCodePoint;
 	for(unsigned long i = 0; i < cch; ++i) {
-		if(Manah::CTextConverter::IsUTF16HighSurrogate(psz[i])
+		if(IsUTF16HighSurrogate(psz[i])
 				&& i < cch - 1
-				&& Manah::CTextConverter::IsUTF16LowSurrogate(psz[i + 1]))
-			nCodePoint = Manah::CTextConverter::DecodeUTF16SurrogatePairToCodePoint(psz + i, cch - i);
+				&& IsUTF16LowSurrogate(psz[i + 1]))
+			nCodePoint = DecodeUTF16SurrogatePairToCodePoint(psz + i, cch - i);
 		else
 			nCodePoint = psz[i];
-		if((i == 0 && CEditView::IsIdentifierStartChar(nCodePoint))
+		if(std::binary_search(listAlphas.begin(), listAlphas.end(), nCodePoint)
+				|| (i == 0 && CEditView::IsIdentifierStartChar(nCodePoint))
 				|| CEditView::IsIdentifierContinueChar(nCodePoint)) {
 			if(nCodePoint >= 0x010000)
 				++i;
@@ -1459,11 +1499,13 @@ inline unsigned long CEditView::IsIdentifier(const wchar_t* psz, unsigned long c
 inline unsigned char CEditView::IsInvalidCharacter(const wchar_t* psz, unsigned long cch) const {
 	assert(psz != 0);
 
+	using namespace Manah::Text;
+
 	if(cch == 0)
 		return 0;
 	else if(cch > 1
-			&& Manah::CTextConverter::IsUTF16HighSurrogate(psz[0])
-			&& Manah::CTextConverter::IsUTF16LowSurrogate(psz[1])) {
+			&& IsUTF16HighSurrogate(psz[0])
+			&& IsUTF16LowSurrogate(psz[1])) {
 		BOOL	b = FALSE;
 		::WideCharToMultiByte(GetDocument()->GetCodePage(), 0, psz, 2, 0, 0, 0, &b);
 		return toBoolean(b) ? 1 : 0;
@@ -1508,9 +1550,9 @@ inline unsigned long CEditView::IsMComment1(const wchar_t* psz, unsigned long cc
 		wchar_t*	pszDummy = new wchar_t[cch];
 		wchar_t*	pszEndStr = new wchar_t[m_FoundationInfo.strCommentEnd1.length() + 1];
 		wcsncpy(pszDummy, psz, cch);
-		Manah::CTextConverter::ToLower(pszDummy, cch);
+		Manah::Text::ToLower(pszDummy, cch);
 		wcscpy(pszEndStr, m_FoundationInfo.strCommentEnd1.c_str());
-		Manah::CTextConverter::ToLower(pszEndStr);
+		Manah::Text::ToLower(pszEndStr);
 		std::wstring	str = wstring(pszDummy, cch);
 		delete[] pszDummy;
 		pszDummy = 0;
@@ -1564,9 +1606,9 @@ inline unsigned long CEditView::IsMComment2(const wchar_t* psz, unsigned long cc
 		wchar_t*	pszDummy = new wchar_t[cch];
 		wchar_t*	pszEndStr = new wchar_t[m_FoundationInfo.strCommentEnd2.length() + 1];
 		wcsncpy(pszDummy, psz, cch);
-		Manah::CTextConverter::ToLower(pszDummy, cch);
+		Manah::Text::ToLower(pszDummy, cch);
 		wcscpy(pszEndStr, m_FoundationInfo.strCommentEnd2.c_str());
-		Manah::CTextConverter::ToLower(pszEndStr);
+		Manah::Text::ToLower(pszEndStr);
 		std::wstring	str = wstring(pszDummy, cch);
 		delete[] pszDummy;
 		pszDummy = 0;
@@ -1602,11 +1644,26 @@ inline bool CEditView::IsNeutralNoAlpha(wchar_t wch) {
 inline unsigned long CEditView::IsNumeric(const wchar_t* psz, unsigned long cch) {
 	assert(psz != 0);
 
+	using namespace Manah::Text;
+
+//	const std::list<unsigned long>&	listAlphas = m_FoundationInfo.listCharsTreatedAsAlpha;
+	unsigned long					cp;
+
 	if(!toBoolean(iswdigit(*psz)))
 		return 0;
 	for(unsigned long i = 1; i < cch; ++i) {
-		if(toBoolean(iswalnum(psz[i])) || psz[i] == L'.')
-			/* Ȃ */;
+		if(i < cch - 1
+				&& IsUTF16HighSurrogate(psz[i])
+				&& IsUTF16LowSurrogate(psz[i + 1]))
+			cp = DecodeUTF16SurrogatePairToCodePoint(psz + i, cch - i);
+		else
+			cp = psz[i];
+		if(psz[i] == L'.'
+//				|| std::binary_search(listAlphas.begin(), listAlphas.end(), cp)
+				|| CEditView::IsIdentifierContinueChar(cp)) {
+			if(cp > 0xFFFF)
+				++i;
+		}
 		else
 			return i;
 	}
