@@ -8,9 +8,15 @@
 
 using namespace Ascension;
 using namespace Manah;
+using namespace Manah::Text;
 using namespace std;
 
 
+// InsertLine AInsertLines ŉ͂郌xB
+// hLg̓ǂݍݑxƂ̌̍ŏ̍s̓I[o[wbhƂ̃g[hɂȂ
+#define MINIMAL_PARSE_LEVEL	LPL_TOKEN
+
+
 // CLineLayoutManager class implementation
 /////////////////////////////////////////////////////////////////////////////
 
@@ -85,10 +91,9 @@ void CLineLayoutManager::DeleteLine(unsigned long iLine) throw(out_of_range) {
 	if(pInfo->m_pPrev != 0 && pInfo->m_pPrev->m_mctToNext == pInfo->m_mctFromPrev)
 		return;
 	while(true) {
-		ParseLineEdgeComments(iLine, pInfo);
-		pInfo->m_bReady = false;
+		ParseLineMultilineCommentContinue(iLine, pInfo);
 		if(pInfo->m_pNext == 0
-				|| !pInfo->m_pNext->m_bEdgeMCommentsCompleted
+				|| (pInfo->m_pNext->m_lplParseLevel < LPL_MULTILINECOMMENT)
 				|| pInfo->m_mctToNext == pInfo->m_pNext->m_mctFromPrev) {	// e͂̍sŏI
 			break;
 		}
@@ -152,10 +157,9 @@ void CLineLayoutManager::DeleteLines(unsigned long iStart, unsigned long iEnd) t
 		return;
 	unsigned long	iLine = iStart;
 	while(true) {
-		ParseLineEdgeComments(iLine, pInfo);
-		pInfo->m_bReady = false;
+		ParseLineMultilineCommentContinue(iLine, pInfo);
 		if(pInfo->m_pNext == 0
-				|| !pInfo->m_pNext->m_bEdgeMCommentsCompleted
+				|| (pInfo->m_pNext->m_lplParseLevel < LPL_MULTILINECOMMENT)
 				|| pInfo->m_mctToNext == pInfo->m_pNext->m_mctFromPrev) {	// e͂̍sŏI
 			break;
 		}
@@ -171,8 +175,9 @@ void CLineLayoutManager::DeleteLines(unsigned long iStart, unsigned long iEnd) t
  */
 CLineLayoutInfo* CLineLayoutManager::GetLine(unsigned long iLine) const {
 	CLineLayoutInfo*	pInfo = GetLineInternal(iLine);
-	if(pInfo != 0 && !pInfo->m_bReady)	// łĂȂ
-		const_cast<CLineLayoutManager*>(this)->ParseLineTokens(iLine, pInfo);
+	if(pInfo != 0 && pInfo->m_lplParseLevel != LPL_FULL)	// łĂȂ
+		const_cast<CLineLayoutManager*>(this)->ParseLineCharacterPositions(iLine, pInfo);
+	assert(pInfo == 0 || pInfo->m_lplParseLevel == LPL_FULL);
 	return pInfo;
 }
 
@@ -210,6 +215,19 @@ CLineLayoutInfo* CLineLayoutManager::GetLineInternal(unsigned long iLine) const
 	return 0;
 }
 
+/**
+ *	ws̉͂ǂ̒xIĂ邩Ԃ
+ *	@param iLine	_s
+ *	@return			̓x
+ */
+LineParseLevel CLineLayoutManager::GetLineParseLevel(unsigned long iLine) const throw(out_of_range) {
+	CLineLayoutInfo*	pInfo = GetLineInternal(iLine);
+
+	if(pInfo == 0)
+		throw out_of_range("Specified line is not found.");
+	return pInfo->m_lplParseLevel;
+}
+
 ///	ő\s̕\Ԃ
 unsigned long CLineLayoutManager::GetMaxDisplayWidth() const {
 	return m_pxMax;
@@ -246,7 +264,14 @@ void CLineLayoutManager::InsertLine(unsigned long iLine) throw(out_of_range) {
 		pPrev->m_pNext = pInfo;
 		pInfo->m_mctFromPrev = pInfo->m_pPrev->m_mctToNext;
 	}
-	ParseLineEdgeComments(iLine, pInfo);
+	if(MINIMAL_PARSE_LEVEL <= LPL_MULTILINECOMMENT)
+		ParseLineMultilineCommentContinue(iLine, pInfo);
+	else if(MINIMAL_PARSE_LEVEL == LPL_TOKEN)
+		ParseLineTokens(iLine, pInfo);
+	else if(MINIMAL_PARSE_LEVEL == LPL_FULL)
+		ParseLineCharacterPositions(iLine, pInfo);
+	else
+		assert(false);
 
 	if(pInfo->m_pNext == 0)
 		return;
@@ -256,13 +281,12 @@ void CLineLayoutManager::InsertLine(unsigned long iLine) throw(out_of_range) {
 	// 㑱̉e󂯂s`FbN
 	while(true) {
 		if(pInfo->m_pNext == 0
-				|| !pInfo->m_pNext->m_bEdgeMCommentsCompleted
+				|| (pInfo->m_pNext->m_lplParseLevel < LPL_MULTILINECOMMENT)
 				|| pInfo->m_mctToNext == pInfo->m_pNext->m_mctFromPrev) {	// e͂̍sŏI
 			break;
 		}
 		pInfo = pInfo->m_pNext;
-		ParseLineEdgeComments(++iLine, pInfo);
-		pInfo->m_bReady = false;
+		ParseLineMultilineCommentContinue(++iLine, pInfo);
 	}
 }
 
@@ -308,8 +332,16 @@ void CLineLayoutManager::InsertLines(unsigned long iStart, unsigned long iEnd) t
 
 	// }s̏XV
 	CLineLayoutInfo*	pInfo = GetLineInternal(iStart);
-	for(unsigned long iLine = iStart; iLine <= iEnd; ++iLine, pInfo = pInfo->m_pNext)
-		ParseLineEdgeComments(iLine, pInfo);
+	for(unsigned long iLine = iStart; iLine <= iEnd; ++iLine, pInfo = pInfo->m_pNext) {
+		if(MINIMAL_PARSE_LEVEL <= LPL_MULTILINECOMMENT)
+			ParseLineMultilineCommentContinue(iLine, pInfo);
+		else if(MINIMAL_PARSE_LEVEL == LPL_TOKEN)
+			ParseLineTokens(iLine, pInfo);
+		else if(MINIMAL_PARSE_LEVEL == LPL_FULL)
+			ParseLineCharacterPositions(iLine, pInfo);
+		else
+			assert(false);
+	}
 
 	// 㑱̉e󂯂s`FbN
 	unsigned long	iLine_ = iEnd;
@@ -318,42 +350,15 @@ void CLineLayoutManager::InsertLines(unsigned long iStart, unsigned long iEnd) t
 		return;
 	while(true) {
 		if(pInfo->m_pNext == 0
-				|| !pInfo->m_pNext->m_bEdgeMCommentsCompleted
+				|| (pInfo->m_pNext->m_lplParseLevel < LPL_MULTILINECOMMENT)
 				|| pInfo->m_mctToNext == pInfo->m_pNext->m_mctFromPrev) {	// e͂̍sŏI
 			break;
 		}
 		pInfo = pInfo->m_pNext;
-		ParseLineEdgeComments(++iLine_, pInfo);
-		pInfo->m_bReady = false;
+		ParseLineMultilineCommentContinue(++iLine_, pInfo);
 	}
 }
 
-/**
- *	ws̉͂IĂ邩Ԃ
- *	@param iLine	_s
- */
-bool CLineLayoutManager::IsLineParseComplete(unsigned long iLine) throw(out_of_range) {
-	CLineLayoutInfo*	pInfo = GetLineInternal(iLine);
-
-	if(pInfo == 0)
-		throw out_of_range("Specified line is not found.");
-	return pInfo->m_bReady;
-}
-
-/**
- *	SĂ̍s̃CAEgXV
- *	@param bReconstruct	xCAEgSč폜č\ǂ
- */
-void CLineLayoutManager::ModifyAllLines(bool bReconstruct) {
-	unsigned long cLines = m_pView->GetDocument()->GetLineCount();
-	if(bReconstruct) {
-		DeleteAllLines();
-		InsertLines(0, cLines - 1);
-	} else {
-		for(unsigned long iLine = 0; iLine < cLines; ++iLine)
-			ModifyLine(iLine);
-	}
-}
 
 /**
  *	w肵s̃CAEgXV
@@ -361,7 +366,7 @@ void CLineLayoutManager::ModifyAllLines(bool bReconstruct) {
  *	@return			es
  *	@exception		iLine ݂Ȃsł <code>out_of_range</code>
  */
-inline unsigned long CLineLayoutManager::ModifyLine(unsigned long iLine) throw(out_of_range) {
+unsigned long CLineLayoutManager::ModifyLine(unsigned long iLine) throw(out_of_range) {
 	CLineLayoutInfo*	pInfo = GetLineInternal(iLine);
 
 	if(pInfo == 0)
@@ -378,12 +383,12 @@ inline unsigned long CLineLayoutManager::ModifyLine(unsigned long iLine) throw(o
 	unsigned long		cxLine = 0;
 
 	pInfo->m_vecWrappedOffsets.clear();
-	pInfo->m_bReady = false;
+	pInfo->m_lplParseLevel = LPL_UNPARSED;
 
 //	hOldFont = m_pView->m_GDIObjects.oMemDC.SelectObject(m_pView->m_GDIObjects.hNormalFont);
 
 //	if(m_pView->m_ModeState.wpmWrapMode == WPM_NONE)
-		ParseLineTokens(iLine, pInfo);
+		ParseLineCharacterPositions(iLine, pInfo);
 /*	else {	// ܂Ԃ̌vZ
 		unsigned long	cxWrap;
 		unsigned int	cxChar;
@@ -416,13 +421,12 @@ inline unsigned long CLineLayoutManager::ModifyLine(unsigned long iLine) throw(o
 	unsigned long	cModifiedLines = 1;
 	while(true) {
 		if(pInfo->m_pNext == 0
-				|| !pInfo->m_pNext->m_bEdgeMCommentsCompleted
+				|| (pInfo->m_pNext->m_lplParseLevel < LPL_MULTILINECOMMENT)
 				|| pInfo->m_mctToNext == pInfo->m_pNext->m_mctFromPrev) {	// e͂̍sŏI
 			break;
 		}
 		pInfo = pInfo->m_pNext;
-		ParseLineEdgeComments(iLine + cModifiedLines, pInfo);
-		pInfo->m_bReady = false;
+		ParseLineMultilineCommentContinue(iLine + cModifiedLines, pInfo);
 		++cModifiedLines;
 	}
 	return cModifiedLines;
@@ -439,14 +443,28 @@ void CLineLayoutManager::ModifyMaxDisplayWidth() {
 	}
 }
 
+/**
+ *	SĂ̍sɑ΂Ďw肵x̉͂蒼Kv邱Ƃʒm
+ *	@param lplDisable	̓x
+ */
+void CLineLayoutManager::NotifyAll(LineParseLevel lplDisable) {
+	CLineLayoutInfo*	pLine = m_pHead;
+
+	while(pLine != 0) {
+		if(pLine->m_lplParseLevel >= lplDisable)
+			pLine->m_lplParseLevel = static_cast<LineParseLevel>(lplDisable - 1);
+		pLine = pLine->m_pNext;
+	}
+}
+
 /**
  *	s̐擪A̕sRg
  *	@param iLine	_sԍ
  *	@param pInfo	s (ȗƍsԍ擾)
- *	@see			ParseLineTokens
  */
-void CLineLayoutManager::ParseLineEdgeComments(unsigned long iLine, CLineLayoutInfo* pInfo /* = 0 */) {
-	if(m_pView->m_FoundationInfo.strCommentStart1.empty() && m_pView->m_FoundationInfo.strCommentStart2.empty())
+void CLineLayoutManager::ParseLineMultilineCommentContinue(unsigned long iLine, CLineLayoutInfo* pInfo /* = 0 */) {
+	if(m_pView->m_FoundationInfo.strCommentStart1.empty()
+			&& m_pView->m_FoundationInfo.strCommentStart2.empty())
 		return;
 
 	if(pInfo == 0)
@@ -456,11 +474,11 @@ void CLineLayoutManager::ParseLineEdgeComments(unsigned long iLine, CLineLayoutI
 	if(pInfo->m_pPrev == 0)
 		pInfo->m_mctFromPrev = MCT_NOTCOMMENT;
 	else {
-		if(!pInfo->m_pPrev->m_bEdgeMCommentsCompleted)	// O̍s͂Ȃ΍ċAIɉ͂
-			ParseLineEdgeComments(iLine - 1, pInfo->m_pPrev);
+		if(pInfo->m_pPrev->m_lplParseLevel < LPL_MULTILINECOMMENT)	// O̍s͂Ȃ΍ċAIɉ͂
+			ParseLineMultilineCommentContinue(iLine - 1, pInfo->m_pPrev);
 		pInfo->m_mctFromPrev = pInfo->m_pPrev->m_mctToNext;
 	}
-	pInfo->m_bEdgeMCommentsCompleted = true;
+	pInfo->m_lplParseLevel = LPL_MULTILINECOMMENT;
 
 	const wchar_t*	pwszLine = m_pView->GetDocument()->GetLine(iLine).c_str();
 	unsigned long	iChar = 0;
@@ -518,11 +536,14 @@ void CLineLayoutManager::ParseLineEdgeComments(unsigned long iLine, CLineLayoutI
 }
 
 /**
- *	ws̉͂sB͍ς݂łΉȂB
+ *	@brief	ws̉͂sB͍ς݂łΉȂ
+ *
  *	CLineLayoutManager ̃NCAg GetLine
  *	gƂŉ͍ς݂̍s擾ł邪AGetLine
  *	ȊÕ\bh͍s͂sȂ̂ŁA
  *	̃\bhŗ\߉͂sƂŃI[owbhɘa邱Ƃł
+ *
+ *	@param iLine	_s
  */
 void CLineLayoutManager::ParseLine(unsigned long iLine) throw(out_of_range) {
 	CLineLayoutInfo*	pInfo = GetLineInternal(iLine);
@@ -530,39 +551,37 @@ void CLineLayoutManager::ParseLine(unsigned long iLine) throw(out_of_range) {
 	if(pInfo == 0)
 		throw out_of_range("Specified line is not found.");
 
-	if(!pInfo->m_bReady)
-		ParseLineTokens(iLine, pInfo);
+	if(pInfo->m_lplParseLevel < LPL_FULL)
+		ParseLineCharacterPositions(iLine, pInfo);
 }
 
 /**
- *	g[N؂oƕʒǔvZs (ParseLineEdgeComments ̓˂)
+ *	ʒǔvZs
  *	@param iLine	_sԍ
  *	@param pInfo	s (ȗƍsԍ擾)
- *	@see			ParseLineEdgeComments
  */
-void CLineLayoutManager::ParseLineTokens(unsigned long iLine, CLineLayoutInfo* pInfo /* = 0 */) {
-	const wchar_t*	pwszLine = m_pView->GetDocument()->GetLine(iLine).c_str();
-	unsigned long	cchLine = m_pView->GetDocument()->GetLineLength(iLine);
-
+void CLineLayoutManager::ParseLineCharacterPositions(unsigned long iLine, CLineLayoutInfo* pInfo /* = 0 */) {
 	if(pInfo == 0)
 		pInfo = GetLineInternal(iLine);
 
-	// O̍s畡sRgĂ邩ǂĂȂ΂Ȃ
-	if(pInfo->m_pPrev != 0 && !pInfo->m_pPrev->m_bEdgeMCommentsCompleted)
-		ParseLineEdgeComments(iLine, pInfo->m_pPrev);
-	pInfo->m_mctFromPrev = (pInfo->m_pPrev != 0) ? pInfo->m_pPrev->m_mctToNext : MCT_NOTCOMMENT;
-	pInfo->m_mctToNext = MCT_NOTCOMMENT;
+	// g[N؂oIĂȂ΂Ȃ
+	if(pInfo->m_lplParseLevel < LPL_TOKEN)
+		ParseLineTokens(iLine, pInfo);
+
+	unsigned long	cchLine = m_pView->GetDocument()->GetLineLength(iLine);
+	unsigned long	iStart = 0;
+	unsigned long	iEnd = 0;
 
-	pInfo->m_listTokens.clear();
 	if(cchLine == 0) {
-		pInfo->m_mctToNext = pInfo->m_mctFromPrev;
 		pInfo->m_cxPixel = 0;
 		pInfo->m_vecCaretPos.clear();
 		pInfo->m_vecCaretPos.push_back(0);
-		pInfo->m_bEdgeMCommentsCompleted = pInfo->m_bReady = true;
+		pInfo->m_lplParseLevel = LPL_FULL;
 		return;
 	}
 
+	TokenList::iterator	itToken = pInfo->m_listTokens.begin();
+	const wchar_t*	pwszLine = m_pView->GetDocument()->GetLine(iLine).c_str();
 	CMinimalDC&		oDC = m_pView->m_GDIObjects.oMemDC;
 	HFONT			hOldFont = oDC.SelectObject(m_pView->m_GDIObjects.hNormalFont);
 	GCP_RESULTSW	gcpr;
@@ -575,10 +594,160 @@ void CLineLayoutManager::ParseLineTokens(unsigned long iLine, CLineLayoutInfo* p
 	ZeroMemory(&gcpr, sizeof(GCP_RESULTSW));
 	gcpr.lStructSize = sizeof(GCP_RESULTSW);
 
-	// ͂ƕʒu
+	while(iEnd < cchLine) {
+		iStart = iEnd;
+		iEnd = /*m_pView->m_ModeState.bResetDirByToken ?*/ (iStart + itToken->m_cchToken) /*: cchLine*/;
+		TTextFoundation&	tf = m_pView->GetTextFoundation(itToken->m_ttType);
+		
+		if(tf.enabled && tf.bold && m_pView->m_FoundationInfo.bDrawBoldCharsClosely)
+			oDC.SetTextCharacterExtra(m_pView->m_layoutInfo.nCharSpan - 1);
+		oDC.SelectObject(m_pView->GetFontForRenderingToken(itToken->m_ttType));
+
+		unsigned long					iLastTab = iStart;
+		CTokenInfo::TSubstringDirection	oDirection = {0, false};
+		SIZE							size;
+
+		itToken->m_nLeftEdge = nOffset;
+		itToken->m_listDirections.push_back(oDirection);
+		while(true) {	// g[Ñ^u͕ʈ
+			const wchar_t*	pwszTab = wcschr(pwszLine + iLastTab, L'\t');
+			unsigned long	cchSubstring;
+			long			cxSubstring;
+
+			if(static_cast<unsigned long>(pwszTab - pwszLine) >= iStart + itToken->m_cchToken)	// ȂƂɂ
+				pwszTab = 0;
+
+			// ^u܂
+			if(pwszTab == 0 || pwszTab - pwszLine != iLastTab) {
+				cchSubstring = (pwszTab != 0) ? pwszTab - pwszLine - iLastTab : iStart - iLastTab + itToken->m_cchToken;
+				cchSubstring = std::min(itToken->m_cchToken - iLastTab + iStart, cchSubstring);
+				gcpr.nGlyphs = cchSubstring;
+				gcpr.lpOrder = arrOrder + iLastTab;
+				oDC.GetCharacterPlacement(pwszLine + iLastTab,
+					cchSubstring, 0, &gcpr, GCP_DIACRITIC | GCP_DISPLAYZWG | GCP_GLYPHSHAPE | GCP_REORDER);
+				oDC.GetTextExtentExPoint(pwszLine + iLastTab,
+					cchSubstring, 0, 0, arrCaretPos + iLastTab + 1, &size);
+				cxSubstring = size.cx;
+				for(unsigned long i = iLastTab; i < iLastTab + cchSubstring; ++i) {
+					arrOrder[i] += iLastTab;
+					arrCaretPos[i + 1] += nOffset;
+				}
+				nOffset += cxSubstring;
+				if(pwszTab == 0)	// g[NI
+					break;
+			}
+
+			// ^u
+			nOffset = m_pView->GetNextTabStop(nOffset, false);
+			iLastTab = pwszTab - pwszLine;
+			arrOrder[iLastTab] = iLastTab;
+			arrCaretPos[iLastTab + 1] = nOffset;
+			++iLastTab;
+		}
+
+		// ̏畔eLXg߂ ([Alef][Bet][Gimel]012 -> [Gimel][Bet][Alef]012)
+		unsigned int	iDesired = iStart;	// ݂̕Ŋ҂鎟̏
+		unsigned long	iRtlStart = 0;		// ŌɉE獶ɂȂʒu
+		for(unsigned long i = iStart; i < iStart + itToken->m_cchToken; ++i) {
+			if(i > 0 && arrOrder[i] == arrOrder[i - 1])	// TQ[gyA
+				continue;
+			else if(arrOrder[i] == iDesired)	// z肳ꂽ
+				iDesired += oDirection.bRightToLeft ? -1 : 1;
+			else {	// ȊO -> ω
+				oDirection.iStartChar = i - iStart;
+				oDirection.bRightToLeft = !oDirection.bRightToLeft;
+				itToken->m_listDirections.push_back(oDirection);
+				iDesired = arrOrder[i] + (oDirection.bRightToLeft ? -1 : 1);
+
+				if(oDirection.bRightToLeft)
+					iRtlStart = i;
+				else {
+					// eLXg̕E獶̃̕Lbgʒu
+					// t] (アƒԕ𓯈ꎋ邪...)
+					int	nSubstringLeft = arrCaretPos[iRtlStart];
+					for(unsigned long j = iRtlStart; j < i; ++j)
+						arrCaretPos[j] = arrCaretPos[i] - arrCaretPos[j] + nSubstringLeft;
+
+					// RTL  LTR  (ア) Oɏo ([Gimel][Bet][Alef]012 -> 012[Gimel][Bet][Alef])
+					unsigned long	cWeakChars = 0;
+					while(i > iStart
+							&& i + cWeakChars < iStart + itToken->m_cchToken
+							&& arrOrder[i + cWeakChars] < arrOrder[i - 1])
+						++cWeakChars;
+					if(cWeakChars != 0) {
+						int	cxRtr = oDC.GetTextExtent(pwszLine + iRtlStart, i - iRtlStart).cx;
+						int	cxLtr = oDC.GetTextExtent(pwszLine + i, cWeakChars).cx;
+						if(i + cWeakChars == cchLine)	// I[܂߂
+							++cWeakChars;
+						for(unsigned long j = iRtlStart; j < i + cWeakChars; ++j)
+							arrCaretPos[j] += (j < i) ? cxLtr : -cxRtr;
+					}
+				}
+			}
+		}
+		if(oDirection.bRightToLeft) {	// E獶̏ꍇ͂ŏ
+			int				nSubstringLeft = arrCaretPos[iRtlStart];
+			unsigned long	iEnd = (iStart + itToken->m_cchToken != cchLine) ?
+								iStart + itToken->m_cchToken : iStart + itToken->m_cchToken + 1;
+
+			for(unsigned long j = iRtlStart; j < iEnd; ++j)
+				arrCaretPos[j] = arrCaretPos[iStart + itToken->m_cchToken] - arrCaretPos[j] + nSubstringLeft;
+		}
+
+		if(tf.enabled && tf.bold && m_pView->m_FoundationInfo.bDrawBoldCharsClosely)
+			oDC.SetTextCharacterExtra(m_pView->m_layoutInfo.nCharSpan);
+		itToken->m_nTextWidth = nOffset - itToken->m_nLeftEdge;
+		++itToken;
+	}
+
+	pInfo->m_cxPixel = nOffset;
+	pInfo->m_vecCaretPos.assign(arrCaretPos, arrCaretPos + cchLine + 1);
+	delete[] arrCaretPos;
+	delete[] arrOrder;
+	oDC.SelectObject(hOldFont);
+
+	// ő\̌vZ (ł͂Ȃ...)
+	if(pInfo == m_pMaxDisplayWidthLine) {
+		ModifyMaxDisplayWidth();
+		m_pView->ModifyScrollInfo(true, false);
+	} else if(pInfo->m_cxPixel > m_pxMax) {
+		m_pxMax = pInfo->m_cxPixel;
+		m_pMaxDisplayWidthLine = pInfo;
+		m_pView->ModifyScrollInfo(true, false);
+	}
+
+	pInfo->m_lplParseLevel = LPL_FULL;
+}
+
+/**
+ *	g[N؂os
+ *	@param iLine	_sԍ
+ *	@param pInfo	s (ȗƍsԍ擾)
+ */
+void CLineLayoutManager::ParseLineTokens(unsigned long iLine, CLineLayoutInfo* pInfo /* = 0 */) {
+	const wchar_t*	pwszLine = m_pView->GetDocument()->GetLine(iLine).c_str();
+	unsigned long	cchLine = m_pView->GetDocument()->GetLineLength(iLine);
+
+	if(pInfo == 0)
+		pInfo = GetLineInternal(iLine);
+
+	// O̍s畡sRgĂ邩ǂĂȂ΂Ȃ
+	if(pInfo->m_pPrev != 0 && (pInfo->m_pPrev->m_lplParseLevel < LPL_MULTILINECOMMENT))
+		ParseLineMultilineCommentContinue(iLine, pInfo->m_pPrev);
+	pInfo->m_mctFromPrev = (pInfo->m_pPrev != 0) ? pInfo->m_pPrev->m_mctToNext : MCT_NOTCOMMENT;
+	pInfo->m_mctToNext = MCT_NOTCOMMENT;
+
+	pInfo->m_listTokens.clear();
+	if(cchLine == 0) {
+		pInfo->m_mctToNext = pInfo->m_mctFromPrev;
+		pInfo->m_lplParseLevel = LPL_TOKEN;
+		return;
+	}
+
+	// 
 	bool	bInComment = pInfo->m_mctFromPrev != MCT_NOTCOMMENT;
 	for(unsigned long iChar = 0; iChar < cchLine; ) {
-		CTokenInfo	oToken;
+		CTokenInfo		oToken;
 
 		// g[N؂o
 		oToken.m_iChar = iChar;
@@ -609,9 +778,9 @@ void CLineLayoutManager::ParseLineTokens(unsigned long iLine, CLineLayoutInfo* p
 			oToken.m_ttType = TT_COMMENT;
 			if(bInComment)
 				pInfo->m_mctToNext = MCT_2;
-		} else if(m_pView->m_layoutInfo.tfTextFoundation[TT_OPERATOR].enabled
+		} /*else if(m_pView->m_layoutInfo.tfTextFoundation[TT_OPERATOR].enabled
 				&& 0 != (oToken.m_cchToken = m_pView->IsOperator(pwszLine + iChar, cchLine - iChar)))	// Zq
-			oToken.m_ttType = TT_OPERATOR;
+			oToken.m_ttType = TT_OPERATOR;*/
 		else if(m_pView->m_layoutInfo.tfTextFoundation[TT_SPACE].enabled
 				&& 0 != (oToken.m_cchToken = m_pView->IsWhiteSpace(pwszLine + iChar, cchLine - iChar)))	// 󔒗ނ
 			oToken.m_ttType = TT_SPACE;
@@ -637,130 +806,25 @@ void CLineLayoutManager::ParseLineTokens(unsigned long iLine, CLineLayoutInfo* p
 				oToken.m_ttType = TT_IDENTIFIER;
 		} else {
 			// TQ[gyÃ`FbN
-			if(CTextConverter::IsUTF16HighSurrogate(pwszLine[iChar])
+			if(IsUTF16HighSurrogate(pwszLine[iChar])
 					&& iChar < cchLine - 1
-					&& CTextConverter::IsUTF16LowSurrogate(pwszLine[iChar + 1]))
+					&& IsUTF16LowSurrogate(pwszLine[iChar + 1]))
 				oToken.m_cchToken = 2;
 			else
 				oToken.m_cchToken = 1;
 			oToken.m_ttType = TT_NORMAL;
 		}
 
-		// ʒuƕ (ȍ~̌vZ͎bIȂ̂ŁA̗]n͏\ɂ)
-		unsigned long					iLastTab = iChar;
-		CTokenInfo::TSubstringDirection	oDirection = {0, false};
-		SIZE							size;
-
-		oDC.SelectObject(m_pView->GetFontForRenderingToken(oToken.m_ttType));
-		oToken.m_nLeftEdge = nOffset;
-		oToken.m_listDirections.push_back(oDirection);
-		while(true) {	// g[Ñ^u͕ʈ
-			const wchar_t*	pwszTab = wcschr(pwszLine + iLastTab, L'\t');
-			unsigned long	cchSubstring;
-			long			cxSubstring;
-
-			if(static_cast<unsigned long>(pwszTab - pwszLine) >= iChar + oToken.m_cchToken)	// ȂƂɂ
-				pwszTab = 0;
-
-			// ^u܂
-			if(pwszTab == 0 || pwszTab - pwszLine != iLastTab) {
-				cchSubstring = (pwszTab != 0) ? pwszTab - pwszLine - iLastTab : iChar - iLastTab + oToken.m_cchToken;
-				cchSubstring = std::min(oToken.m_cchToken - iLastTab + iChar, cchSubstring);
-				gcpr.nGlyphs = cchSubstring;
-				gcpr.lpOrder = arrOrder + iLastTab;
-				oDC.GetCharacterPlacement(pwszLine + iLastTab,
-					cchSubstring, 0, &gcpr, GCP_DIACRITIC | GCP_DISPLAYZWG | GCP_GLYPHSHAPE | GCP_REORDER);
-				oDC.GetTextExtentExPoint(pwszLine + iLastTab,
-					cchSubstring, 0, 0, arrCaretPos + iLastTab + 1, &size);
-				cxSubstring = size.cx;
-				for(unsigned long i = iLastTab; i < iLastTab + cchSubstring; ++i) {
-					arrOrder[i] += iLastTab;
-					arrCaretPos[i + 1] += nOffset;
-				}
-				nOffset += cxSubstring;
-				if(pwszTab == 0)	// g[NI
-					break;
-			}
-
-			// ^u
-			nOffset = m_pView->GetNextTabStop(nOffset, false);
-			iLastTab = pwszTab - pwszLine;
-			arrOrder[iLastTab] = iLastTab;
-			arrCaretPos[iLastTab + 1] = nOffset;
-			++iLastTab;
-		}
-
-		// ̏畔eLXg߂ ([Alef][Bet][Gimel]012 -> [Gimel][Bet][Alef]012)
-		unsigned int	iDesired = iChar;	// ݂̕Ŋ҂鎟̏
-		unsigned long	iRtlStart = 0;		// ŌɉE獶ɂȂʒu
-		for(unsigned long i = iChar; i < iChar + oToken.m_cchToken; ++i) {
-			if(i > 0 && arrOrder[i] == arrOrder[i - 1])	// TQ[gyA
-				continue;
-			else if(arrOrder[i] == iDesired)	// z肳ꂽ
-				iDesired += oDirection.bRightToLeft ? -1 : 1;
-			else {	// ȊO -> ω
-				oDirection.iStartChar = i - iChar;
-				oDirection.bRightToLeft = !oDirection.bRightToLeft;
-				oToken.m_listDirections.push_back(oDirection);
-				iDesired = arrOrder[i] + (oDirection.bRightToLeft ? -1 : 1);
-
-				if(oDirection.bRightToLeft)
-					iRtlStart = i;
-				else {
-					// eLXg̕E獶̃̕Lbgʒu
-					// t] (アƒԕ𓯈ꎋ邪...)
-					int	nSubstringLeft = arrCaretPos[iRtlStart];
-					for(unsigned long j = iRtlStart; j < i; ++j)
-						arrCaretPos[j] = arrCaretPos[i] - arrCaretPos[j] + nSubstringLeft;
-
-					// RTL  LTR  (ア) Oɏo ([Gimel][Bet][Alef]012 -> 012[Gimel][Bet][Alef])
-					unsigned long	cWeakChars = 0;
-					while(i > iChar
-							&& i + cWeakChars < iChar + oToken.m_cchToken
-							&& arrOrder[i + cWeakChars] < arrOrder[i - 1])
-						++cWeakChars;
-					if(cWeakChars != 0) {
-						int	cxRtr = oDC.GetTextExtent(pwszLine + iRtlStart, i - iRtlStart).cx;
-						int	cxLtr = oDC.GetTextExtent(pwszLine + i, cWeakChars).cx;
-						if(i + cWeakChars == cchLine)	// I[܂߂
-							++cWeakChars;
-						for(unsigned long j = iRtlStart; j < i + cWeakChars; ++j)
-							arrCaretPos[j] += (j < i) ? cxLtr : -cxRtr;
-					}
-				}
-			}
-		}
-		if(oDirection.bRightToLeft) {	// E獶̏ꍇ͂ŏ
-			int				nSubstringLeft = arrCaretPos[iRtlStart];
-			unsigned long	iEnd = (iChar + oToken.m_cchToken != cchLine) ?
-								iChar + oToken.m_cchToken : iChar + oToken.m_cchToken + 1;
-
-			for(unsigned long j = iRtlStart; j < iEnd; ++j)
-				arrCaretPos[j] = arrCaretPos[iChar + oToken.m_cchToken] - arrCaretPos[j] + nSubstringLeft;
-		}
-
-		oToken.m_nTextWidth = nOffset - oToken.m_nLeftEdge;
 		iChar += oToken.m_cchToken;
 		pInfo->m_listTokens.push_back(oToken);
 	}
 
-	pInfo->m_cxPixel = nOffset;
-	pInfo->m_vecCaretPos.assign(arrCaretPos, arrCaretPos + cchLine + 1);
-	delete[] arrCaretPos;
-	delete[] arrOrder;
-	oDC.SelectObject(hOldFont);
+	pInfo->m_lplParseLevel = LPL_TOKEN;
+}
 
-	// ő\̌vZ
-	if(pInfo == m_pMaxDisplayWidthLine) {
-		ModifyMaxDisplayWidth();
-		m_pView->ModifyScrollInfo(true, false);
-	} else if(pInfo->m_cxPixel > m_pxMax) {
-		m_pxMax = pInfo->m_cxPixel;
-		m_pMaxDisplayWidthLine = pInfo;
-		m_pView->ModifyScrollInfo(true, false);
-	}
-
-	pInfo->m_bEdgeMCommentsCompleted = pInfo->m_bReady = true;
+void CLineLayoutManager::ReconstructAll() {
+	DeleteAllLines();
+	InsertLines(0, m_pView->GetDocument()->GetLineCount() - 1);
 }
 
 /* [EOF] */
\ No newline at end of file
