@@ -64,6 +64,18 @@ enum TextType {
 	TEXTTYPE_NUMBER
 };
 
+///	s̉̓x (CLineLayoutInfo Q)
+enum LineParseLevel {
+	///	S͂ĂȂ
+	LPL_UNPARSED,
+	///	O̍sւ̕sRgp܂ŉ͍ς
+	LPL_MULTILINECOMMENT,
+	///	g[N܂ŉ͍ς
+	LPL_TOKEN,
+	///	ʒuƌꏇ܂ (܂Ԃ܂ <- ) ͍ς
+	LPL_FULL
+};
+
 
 // CTokenInfo class definition
 /////////////////////////////////////////////////////////////////////////////
@@ -130,16 +142,20 @@ typedef std::list<CTokenInfo>	TokenList;
 /////////////////////////////////////////////////////////////////////////////
 
 /**
- *	\brief	es̃CAEgۑ
+ *	@brief	es̃CAEgۑ
  *
  *	CLineLayoutManager::GetLine() 擾łA_s̃CAEgێB
  *	O̍sƂ̊Ԃ̕sRǧpAg[ÑXgAẽLbgʒuA
  *	ubN}[NsA܂Ԃɕ`悷̂ɕKvȕȂǂƂĎB
  *
- *	eȏ CLineLayoutManager NXsBCAEg
- *	úvAusRĝ͍݉ς݁vAug[N܂ŉ͍ς݁v
- *	3̏ԂÂ߂Ɋes̉͂͒xB CEditView
- *	ANZXƂɂ́ug[N܂ŉ͍ς݁vł邱Ƃۏ؂B
+ *	eȏ CLineLayoutManager NXsB
+ *	s͂̏󋵂 LineParseLevel 񋓂Ɏ4iKB
+ *	̒iKɕĂ̂͌̂߂łB CEditView
+ *	ANZXƂɂ͈ʒu܂ŊSɉ͍ς݂ł邱Ƃۏ؂B
+ *	NCAg̉͒iKlKv̂ CLineLayoutManager::NotifyAll
+ *	\bhĂяoƂłBႦ΃r[̃tHgωƂA
+ *	SẲ͂蒼Kv͖B̏ꍇALPL_FULL
+ *	w肵ă\bhĂяoΈʒu񂾂𖳌ɂ邱Ƃł
  */
 class CLineLayoutInfo {
 	friend class CLineLayoutManager;
@@ -148,8 +164,7 @@ class CLineLayoutInfo {
 private:
 	CLineLayoutInfo() : m_pNext(0), m_pPrev(0),
 			m_mctFromPrev(MCT_NOTCOMMENT), m_mctToNext(MCT_NOTCOMMENT),
-			m_bBookmarked(false), m_cxPixel(0), m_dwParam(0),
-			m_bReady(false), m_bEdgeMCommentsCompleted(false) {
+			m_bBookmarked(false), m_cxPixel(0), m_dwParam(0), m_lplParseLevel(LPL_UNPARSED) {
 	}
 
 	// \bh
@@ -191,10 +206,9 @@ private:
 	unsigned long				m_cxPixel;				// ܂Ԃɕ`悷̂ɕKvȕ̕ (܂ԂȂꍇ̂ݗL)
 	DWORD						m_dwParam;				// AvP[V`̒l
 
-	bool						m_bReady;					// s̉͂Ă邩
-	bool						m_bEdgeMCommentsCompleted;	// s̐擪Ɩ̕sRg񂪎擾ς݂
-	TokenList					m_listTokens;				// g[ÑXg
-	std::vector<int>			m_vecCaretPos;				// e̍[̈ʒu
+	LineParseLevel				m_lplParseLevel;		// s̉̓x
+	TokenList					m_listTokens;			// g[ÑXg
+	std::vector<int>			m_vecCaretPos;			// e̍[̈ʒu
 };
 
 
@@ -225,17 +239,19 @@ public:
 	void				DeleteLine(unsigned long iLine) throw(out_of_range);
 	void				DeleteLines(unsigned long iStart, unsigned long iEnd) throw(out_of_range);
 	CLineLayoutInfo*	GetLine(unsigned long iLine) const;
+	LineParseLevel		GetLineParseLevel(unsigned long iLine) const throw(out_of_range);
 	unsigned long		GetMaxDisplayWidth() const;
 	void				InsertLine(unsigned long iLine) throw(out_of_range);
 	void				InsertLines(unsigned long iStart, unsigned long iEnd) throw(out_of_range);
-	bool				IsLineParseComplete(unsigned long iLine) throw(out_of_range);
-	void				ModifyAllLines(bool bReconstruct);
 	unsigned long		ModifyLine(unsigned long iLine) throw(out_of_range);
+	void				NotifyAll(LineParseLevel lplDisable);
 	void				ParseLine(unsigned long iLine) throw(out_of_range);
+	void				ReconstructAll();
 protected:
 	CLineLayoutInfo*	GetLineInternal(unsigned long iLine) const;
 	void				ModifyMaxDisplayWidth();
-	void				ParseLineEdgeComments(unsigned long iLine, CLineLayoutInfo* pInfo = 0);
+	void				ParseLineCharacterPositions(unsigned long iLine, CLineLayoutInfo* pInfo = 0);
+	void				ParseLineMultilineCommentContinue(unsigned long iLine, CLineLayoutInfo* pInfo = 0);
 	void				ParseLineTokens(unsigned long iLine, CLineLayoutInfo* pInfo = 0);
 
 	// f[^o
