@@ -40,7 +40,7 @@ public:
 	unsigned long			GetViewCount() const;
 	void					RemoveView(CView* pWnd);
 	void					SetupAllViews(LPARAM lParam = 0L);
-	void					UpdateAllViews(CView* pSender, LPARAM lParam = 0L, void* pHint = 0);
+	void					UpdateAllViews(CObject* pSender, LPARAM lParam = 0L, void* pHint = 0);
 	// hLg
 	tstring			GetExtensionName() const;
 	const tstring&	GetPathName() const;
@@ -83,7 +83,7 @@ public:
 	CDocument*		GetDocument() const;
 protected:
 	virtual void	OnDocumentSetup(LPARAM lHint = 0);
-	virtual void	OnUpdate(CView* pSender, LPARAM lHint, void* pHint);
+	virtual void	OnUpdate(CObject* pSender, LPARAM lHint, void* pHint);
 
 	// f[^o
 protected:
@@ -241,7 +241,7 @@ inline void CDocument::SetupAllViews(LPARAM lParam /* = 0L */) {
 	}
 }
 
-inline void CDocument::UpdateAllViews(CView* pSender, LPARAM lParam /* = 0L */, void* pHint /* = 0 */) {
+inline void CDocument::UpdateAllViews(CObject* pSender, LPARAM lParam /* = 0L */, void* pHint /* = 0 */) {
 	AssertValid();
 	
 	std::list<CView*>::iterator	it = m_listViews.begin();
@@ -285,7 +285,7 @@ inline CDocument* CView::GetDocument() const {
 inline void CView::OnDocumentSetup(LPARAM lHint /* = 0 */) {
 }
 
-inline void CView::OnUpdate(CView* pSender, LPARAM lHint, void* pHint) {
+inline void CView::OnUpdate(CObject* pSender, LPARAM lHint, void* pHint) {
 	InvalidateRect(0);
 }
 
